import csv
import functools
import os
import click
import logging
import json
import sys

from pydash import is_empty
from pydash.collections import find
import pandas as pd
from ec.gleanerio.gleaner import getSitemapSourcesFromGleaner, getGleaner
from ec.objects.utils import parts_from_urn
from ec.datastore import s3
from ec.logger import config_app
import datetime
import pytz

log = config_app()

class EcConfig(object):
    """ Parameters that might be common to commands"""
    def __init__(self, cfgfile=None, s3server=None, s3bucket=None, upload=None, output=None,debug=False):
        if cfgfile:
            s3endpoint, bucket, glnr = getGleaner(cfgfile)
            minio = glnr.get("minio")
            # passed paramters override the config parameters
            self.s3server = s3server if s3server else s3endpoint
            self.bucket = s3bucket if s3bucket else bucket
            self.glnr = glnr
        else:
            self.s3server = s3server
            self.bucket = s3bucket
        self.output= output
        self.upload = upload
        self.debug = debug

    # lets put checks as methods in here.
    # that way some checks if we can connect can be done in one place
    def hasS3(self) -> bool:
         if   ( is_empty(self.s3server) or is_empty(self.bucket) ):
             log.fatal(f" must provide a gleaner config or (s3endpoint and s3bucket)]")
             raise Exception("must provide a gleaner config or (s3endpoint and s3bucket)]")
         return True
    def hasS3Upload(self) -> bool:
         if  not self.upload and ( is_empty(self.s3server) or is_empty(self.bucket) ):
             log.fatal(f" must provide a gleaner config or (s3endpoint and s3bucket)]")
             raise Exception("must provide a gleaner config or (s3endpoint and s3bucket)]")
         return True

def common_params(func):
    @click.option('--cfgfile', help='gleaner config file', type=click.Path(exists=True))
    @click.option('--s3server', help='s3 server address')
    @click.option('--s3bucket', help='s3 bucket')
    @click.option('--upload', help='upload to s3 bucket', default=True)
    @click.option('--output', help='dump to file', type=click.File('wb'))
    @click.option('--debug', default=False, envvar='REPO_DEBUG')
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        return func(*args, **kwargs)
    return wrapper

@click.group()
@common_params
def cli(cfgfile, s3server, s3bucket, upload, output, debug):
    obj = EcConfig(cfgfile, s3server, s3bucket, upload, output, debug)

@cli.command()
@click.option('--path', help='Path to source',)
@click.option('--source', help='One or more repositories (--source a --source b)', multiple=True)
@common_params
def count(cfgfile, s3server, s3bucket, upload, output, debug, source, path):
    ctx = EcConfig(cfgfile, s3server, s3bucket, upload, output, debug)
    s3Minio = s3.MinioDatastore(ctx.s3server, None)
    ctx.hasS3()

    if path:
        count = s3Minio.countPath(ctx.bucket, path)
        res = f"Count for path {path}: {count}"
        sys.stdout.write(json.dumps(res, sort_keys=True, indent=4))
    elif source:
        counts = list()
        for s in source:
            path = f"/{s3Minio.paths.get('summon')}/{s}"
            count = s3Minio.countPath(ctx.bucket, path)
            res = f"Count for source {s}: {count}"
            counts.append(res)
        sys.stdout.write(json.dumps(counts, sort_keys=True, indent=4))
    elif cfgfile:
        sources = getSitemapSourcesFromGleaner(cfgfile)
        counts = list()
        for s in sources:
            path = f"/{s3Minio.paths.get('summon')}/{s.get('name')}"
            count = s3Minio.countPath(ctx.bucket, path)
            res = f"Count for source {s.get('name')}: {count}"
            counts.append(res)
        sys.stdout.write(json.dumps(counts, sort_keys=True, indent=4))
    else:
        message = "Please provide path to source. E.g. --path milled/iris or a config and optional sources"
        log.fatal(message)
        raise Exception(message)
    return

@cli.command()
@click.option('--source', help='A repository', required=True)
@common_params
def urls(cfgfile, s3server, s3bucket, upload, output, debug, source):
    """Retreive the URL harvested for a give bucket. T
    There may be duplicate URL's, if an HTML has more than one JSONLD"""
    ctx = EcConfig(cfgfile, s3server, s3bucket, upload, output, debug)
    s3Minio = s3.MinioDatastore(ctx.s3server, None)
    ctx.hasS3()

    res = s3Minio.listSummonedUrls(ctx.bucket, source)
    res = pd.DataFrame(res).to_csv(index=False, quoting=csv.QUOTE_NONNUMERIC)
    sys.stdout.write(res)
    if output:
        log.info(f"report for {source} appended to file")
        output.write(bytes(res, 'utf-8'))
    if upload:
        s3Minio.putReportFile(ctx.bucket, source, "bucketutil_urls.csv", res)
    return

@cli.command()
@click.option('--urn', help='One or more urns (--urn a --urn b)', multiple=True, required=True)
@common_params
def download(cfgfile, s3server, s3bucket, upload, output, debug, urn):
    """For a given URN(s), download the files and the Metadata"""
    ctx = EcConfig(cfgfile, s3server, s3bucket, upload, output, debug)
    s3Minio = s3.MinioDatastore(ctx.s3server, None)
    ctx.hasS3()

    for sha in urn:
        # need to parse the source and actual sha/id out of the urn
        # and then do the path and creat path
        parts = parts_from_urn(sha)
        mypath = parts.get('source')
        id = parts.get('id')
        if not os.path.isdir(mypath):
            os.makedirs(mypath)
        outFileName = f"{mypath}/{id}.jsonld"
        log.info(outFileName)
        outFile = open(outFileName, "wb")
        o = s3Minio.getJsonLD(ctx.bucket, mypath, id)
        outFile.write(o)
        outFile.close()
        outFileName = f"{mypath}/{id}.jsonld.meta.txt"
        log.info(outFileName)
        outFile = open(outFileName, "wb")
        o = s3Minio.getJsonLDMetadata(ctx.bucket, mypath, id)
        outFile.write( bytes(json.dumps(o), 'utf8'))
        outFile.close()
    return

@cli.command()
@click.option('--url', help='the X-Amz-Meta-Url in metadata', required=True)
@click.option('--milled', help='include milled', default=False)
@click.option('--summoned', help='include summon only', default=True)
@common_params
def sourceurl(cfgfile, s3server, s3bucket, upload, output, debug, url, summon, milled):
    """ for a given url, find the sha of the file"""
    ctx = EcConfig(cfgfile, s3server, s3bucket, upload, output, debug)
    ctx.hasS3()
    s3Minio = s3.MinioDatastore(ctx.s3server, None)
    paths = list()
    o_list = list()
    if summon:
        paths = list(s3Minio.listPath(ctx.bucket, 'summoned/', recursive=False))
    if milled:
        paths.extend(list(s3Minio.listPath(ctx.bucket, 'milled/', recursive=False)))
    for path in paths:
        try:
            jsonlds = s3Minio.listPath(ctx.bucket, path.object_name, include_user_meta=True)
            objs = map(lambda f: s3Minio.s3client.stat_object(f.bucket_name, f.object_name), jsonlds)
            o_list.extend(list(filter(lambda f: f.metadata.get('X-Amz-Meta-Url') == url, objs)))
        except Exception as e:
            logging.error(e)

    if len(o_list)>0:
        for o in o_list:
            outFileName = f"{o.metadata.get('X-Amz-Meta-Uniqueid')}.jsonld"
            log.info(outFileName)
            outFile = open(outFileName, "wb")
            s3ObjectInfo = {"bucket_name": ctx.bucket, "object_name": o.object_name}
            resp = s3Minio.getFileFromStore(s3ObjectInfo)
            outFile.write(resp)
            outFile.close()
            outFileName = f"{o.metadata.get('X-Amz-Meta-Uniqueid')}.jsonld.meta.txt"
            log.info(outFileName)
            outFile = open(outFileName, "wb")
            s3ObjectInfo = {"bucket_name": ctx.bucket, "object_name": o.object_name}
            tags = s3Minio.getFileMetadataFromStore(s3ObjectInfo)
            outFile.write(bytes(json.dumps(tags, indent=4), 'utf8'))
            outFile.close()
    else:
        log.info(f"Nothing has been found for url: ", url)
    return


@cli.command()
@click.option('--source', help='One or more repositories (--source a --source b)', multiple=True)
@common_params
def duplicateurls(cfgfile, s3server, s3bucket, upload, output, debug, source):
    """ Find Possible Duplicates based on the URL that was used to summon JSONLD.
    Note, the may be ok, if a given URL has more than one JSONLD in the HTML.
    """
    ctx = EcConfig(cfgfile, s3server, s3bucket, upload, output, debug)
    ctx.hasS3()
    s3Minio = s3.MinioDatastore(ctx.s3server, None)
    df = s3Minio.listDuplicateUrls(ctx.bucket, source)
    try:
        if len(df) <= 0:
            logging.info(f"No object has been found")
            return

        df = df[df['Url Duplicates'] > 1]
        if len(df) <= 0:
            logging.info(f"No duplicate has been found")
            return

        df['Example'] = df.apply(lambda f: f['Name'] + ', ' + str(f['Date']), axis=1)
        df = df.groupby(['Source', 'Url'], group_keys=True, dropna=False) \
            .agg({'Name': 'count', 'Example': lambda x: x.iloc[0:5].tolist()}).reset_index()
        df = df.rename(columns={'Name': 'Duplicates'})

        res = df.to_csv(index=False)
        sys.stdout.write(res)
        if output:
            output.write(bytes(res, 'utf-8'))
        if upload:
            s3Minio.putReportFile(ctx.bucket, "all", "bucketutil_dupliactes.csv", res)
    except Exception as e:
        logging.info(e)
    return 0

@cli.command()
@click.option('--source', help='One or more repositories (--source a --source b)', multiple=True)
@common_params
def stats(cfgfile, s3server, s3bucket, upload, output, debug, source):
    ctx = EcConfig(cfgfile, s3server, s3bucket, upload, output, debug)
    ctx.hasS3()
    s3Minio = s3.MinioDatastore(ctx.s3server, None)
    sources = getSitemapSourcesFromGleaner(cfgfile)
    sources = list(filter(lambda source: source.get('active'), sources))
    sources = list(map(lambda r: r.get('name'), sources))
    sources_to_run = source  # optional if null, run all

    pathMilled = f"/{s3Minio.paths.get('milled')}/"
    pathSummon = f"/{s3Minio.paths.get('summon')}/"
    stats = {'milled': {'total': 0, 'repo': {}}, 'summon': {'total': 0, 'repo': {}}}
    for repo in sources:
        if sources_to_run is not None and len(sources_to_run) > 0:
            if not find(sources_to_run, lambda x: x == repo):
                continue
        countMilled = s3Minio.countPath(ctx.bucket, pathMilled + repo)
        countSummon = s3Minio.countPath(ctx.bucket, pathSummon + repo)
        if countMilled > 0:
            stats['milled']['repo'] |= {repo: countMilled}
            stats['milled']['total'] += countMilled
        if countSummon > 0:
            stats['summon']['repo'] |= {repo: countSummon}
            stats['summon']['total'] += countSummon
    res = json.dumps(stats, sort_keys = True, indent = 4)
    sys.stdout.write(res)
    if output:  # just append the json files to one filem, for now.
        output.write(bytes(res, 'utf-8'))
    if upload:
        s3Minio.putReportFile(ctx.bucket, "all", "bucketutil_stats.json", res)

@cli.command()
@click.option('--source', help='One or more repositories (--source a --source b)', multiple=True)
@common_params
def cullurls(cfgfile, s3server, s3bucket, upload, output, debug, source):
    """ for a given url, find the sha of the file"""
    ctx = EcConfig(cfgfile, s3server, s3bucket, upload, output, debug)
    ctx.hasS3()
    s3Minio = s3.MinioDatastore(ctx.s3server, None)
    utc = pytz.UTC
    df = s3Minio.listDuplicateUrls(ctx.bucket, source)
    try:
        if len(df) <= 0:
            logging.info(f"No object has been found")
            return

        df = df[df['Url Duplicates'] > 1]
        if len(df) <= 0:
            logging.info(f" Nothing to cull")
            return
        # delete duplicated objects that are older than 7 days ago
        df = df[df['Date'] < utc.localize(datetime.datetime.now() - datetime.timedelta(days=7))]
        removeObjectList = df.get('Name').values.tolist()
        for r in removeObjectList:
            s3Minio.removeObject(ctx.bucket, str(r))
            logging.info('Removed object: ' + str(r))
    except Exception as e:
        logging.info(e)
    return

if __name__ == '__main__':
    res = cli()



import hashlib
import tempfile
from os import path
import uuid

import uuid as uuid
from rocrate.rocrate import ROCrate
from rocrate.model.person import Person
from rocrate.model.softwareapplication import SoftwareApplication
from rocrate.model.contextentity import ContextEntity
from rocrate.model.data_entity import DataEntity
from rocrate.model.computationalworkflow import ComputationalWorkflow

from rocrate.model.dataset import Dataset
from rocrate.model.metadata import Metadata
from rocrate.model.file import File
from rocrate.model.file_or_dir import FileOrDir

from rocrateValidator import validate
from ec.datastore.s3 import MinioDatastore

## Fair Signposting
# https://pypi.org/project/signposting/
# https://signposting.org/FAIR/
# https://fair-impact.eu/enabling-fair-signposting-and-ro-crate-contentmetadata-discovery-and-consumption
#######

# issue https://github.com/earthcube/GeoCODES-Metadata/issues/5
# logic that might be utilize
# https://github.com/MBcode/ec/blob/8255d7c1312faca721000535b85cd1e22470a73e/ec.py#L2472

##### THOUGHTS #####
# creating a crate
# it will have multiple objects that are in a collection.
# for a collection on the client, we will ask the user to select the distribution to be utilize
# that will be added to the rocrate

# create named crate
# add creator
# for each 'dataset' add to crate as dietibution
# for each tool add to create a worklflow

# then create a friggin tool that read the firrig crate from frigging s3,
# now, that it is downloaded, head the distribitions.
# create tool to download the frigging links from the firring distributions.
# then tell the tool where the files are, and execute the tool
#  how frigging hard can that workflow be to understand.
#  linking tool to the datasets to the tool will be the fun part



#####
# add distribution... up here to prevent some linting errors
######
"""
 web etities: https://www.researchobject.org/ro-crate/1.1/data-entities.html#web-based-data-entities
 {
     "@id": "https://zenodo.org/record/3541888/files/ro-crate-1.0.0.pdf",
     "@type": "File",
     "name": "RO-Crate specification",
     "contentSize": "310691",
     "description": "RO-Crate specification",
     "encodingFormat": "application/pdf"
   }
 """

# option 2: it's a dataset frament

"""
  {
    "@id": "lots_of_little_files/",
    "@type": "Dataset",
    "name": "Too many files",
    "description": "This directory contains many small files, that we're not going to describe in detail.",
    "distribution": {"@id": "http://example.com/downloads/2020/lots_of_little_files.zip"}
  }
"""

# option 3: it's a data download

"""
 {
    "@id": "http://example.com/downloads/2020/lots_of_little_files.zip",
    "@type": "DataDownload",
    "encodingFormat": "application/zip",
    "contentSize": "82818928"
  }
"""



# this tool will read information from an s3 bucket, or a local zip crate
def readDecoderRoCrate(crate, bucket="gleaner", s3endpoint=None) -> ROCrate:
    if (s3endpoint is not None):
        client = MinioDatastore(s3endpoint)
        data = client.getRoCrateFile(crate, bucket=bucket)
        p = path.join(tempfile.gettempdir(), crate)
        with open(p, mode='wb') as f:
            f.write(crate)
        crate = p
    crate = ROCrate(crate)  # or ROCrate('exp_crate.zip')
    return crate


def RoCrateToGraph(crate, endpoint=None):
    crate = ROCrate(crate)  # or ROCrate('exp_crate.zip')
    # convert to graph
    # upload to blazegraph, ignore... but let us search for what is being linked to
    pass


## routines that should accept the Schema.org JSONLd object for RO Crate api's
## and do some courtesy conversion.
### really thiw will need to be done on the javascript side, but
## prototype it here
def _createIdentifier(proposedid):
    if proposedid is not None and isinstance(proposedid, str):

        m = hashlib.md5(proposedid.encode(encoding='UTF-8', errors='strict')).hexdigest()
    else:
        # random id
        m = uuid.uuid4().hex
    return f"uuid:{m}"

DATASET__DATA_DOWNLOAD = '@DataDownload'  # distribution
DATASET__DATASET = '@Dataset'  # Dataset
DATASET__WEB_ENTITY = '@File'  # url, aka web link
# DATASET__WEB_SERVICE = '@SERVICE'   #how do we do a service?

## we do not want to override the methods, we want to add methods
# crates are a bit of a pain because they use disk layout, rather than just metadata file

class SosRocrate(ROCrate):
    """ Science on Schema(SOS) ROCrate Object.
    ROCrate plus methods to easily add SOS information to an ROCrate

    USE:
     [Consuming an RO-Crate](https://github.com/ResearchObject/ro-crate-py#consuming-an-ro-crate):
    use SosRocrate instead of RoCrate

    """
    def nameCrate(self, aName):
        self.nameCrate(aName)

    # this is the creator of the crate, it should be a sos_person
    # probably not correct, but it's a start
    def addSosCreator(self, crate, username):
        properties = {"name": username}

        creator = Person(crate, identifier=_createIdentifier(username), properties=properties)
        self.creator=creator

    # we are  of the crate, it should be a sos_person
    # probably not correct, but it's a start
    def addSosPublisher(self, crate, identifier=None, name=None):
        properties = {"name": name}
        if identifier is None:
            identifier = name
        creator = Person(crate, identifier=_createIdentifier(identifier), properties=properties)
        self.publisher = creator

        # pption 1 it's a file




    def addSosDistribution(self, crate, dataset_jsonld, distribution_to_add=None, distType=DATASET__DATA_DOWNLOAD):
        aurl = dataset_jsonld.get('url')
        name = dataset_jsonld.get('name')
        if distribution_to_add is None:
            #kw = {"fetch_remote": fetch_remote, "validate_url": validate_url}
            kw = {"name": name}
            self.add_file(aurl, properties=kw)
        elif distType == DATASET__DATA_DOWNLOAD :
            pass
        elif distType == DATASET__DATASET:
            self.add_dataset(source=dataset_jsonld)

    def addSosServicesAsEntity(self,crate, url=None, name=None):
        kw = {"name": name}
        self.add_file(source=url, properties=kw)
        pass

    def addSosURL(self, url=None, name=None):
        properties = {"name":name}
        self.add_file(source=url, properties=properties)
        #self.add_file(url, identifier=CreateIdentifier(url), properties=kw)

# two possible ways to add the whole JSONLD.
    # 1. File: url plus the jsonld
    # 2. Dataset (aka directory): url to geocodes, plus jsonld

    # these need to check that these are datasets ;)
    def addSosDatasetAsFile(self, dataset_jsonld,  url):
        properties = dataset_jsonld
        self.add_file(source=url, properties=properties)
### humm https://github.com/ResearchObject/ro-crate-py#adding-entities-with-an-arbitrary-type
    def addSosDatasetAsDataset(self, dataset_jsonld, urn, distType=DATASET__DATASET):
        self.add_dataset(dest_path=urn, properties=dataset_jsonld)


    # this does not get added to hasParts... aka does not get Identifieed as a Dataset
    def addSosDatasetAsCtxEntity(self, dataset_jsonld, urn, distType=DATASET__DATASET):
        """Not useful. this does not get added to hasParts... aka does not get Identifieed as a Dataset """
        datasetCtxEntity =ContextEntity(self, identifier=urn, properties=dataset_jsonld)

        self.add(datasetCtxEntity)
    def addSosContact(self, sos_contact):
            pass



import logging

import pyld.jsonld
import rdflib

from ec.sos_json.rdf import get_rdfgraph

from ec.notebook import utils
from ec.sos_json.utils import formatted_jsonld


class EcObjectManager():
    datastore = None
    bucket = None

    graphendpoint = None

    json_obj = None
    graph_obj = None

    def __init__(self, graphsendpoint, datastore, bucket):
        self.graphendpoint = graphsendpoint
        self.datastore = datastore
        self.bucket = bucket

    # Thinking this could be
    #  form= jsonld
    # forrm = rdflib
    # need to add frame, compact, frame queries type.
    # form = somthing easy to add to a pandas dataframe?
    def getFromStore(self, urn, source='graph', form='rdflib') -> str:

        if source == 'datastore':
            logging.debug('datastore')
            self.json_obj = self._getFromDatastore(urn)
        else:
            logging.debug('default, graph')
            self.graph_obj = self._getFromGraphstore(urn)
        #####

        if form == 'jsonld':
            logging.debug('jsonld')
            if self.json_obj is not None:
                return self.json_obj
            else:
                logging.debug(' convert from graph to jsonld')
                jld_str = self.graph_obj.serialize(format="json-ld")
                return formatted_jsonld(jld_str)
                # not sure how nquads will be handled. know it can only be loaded as RDFlb Dataset...
                # options={"format":  'application/n-quads',"processingMode": 'json-ld-1.1'}


        elif form == 'rdflib':
            logging.debug('rdflib')
            if self.graph_obj is not None:
                return self.graph_obj
            else:
                logging.debug('convert form jsonld to graph')
                options = {"processingMode": 'json-ld-1.1'}
                rdf_str = pyld.jsonld.to_rdf(self.json_obj, options=options)
                g = rdflib.Graph()
                g.parse(data=rdf_str)
                return g

        else:
            logging.debug('default, pyld')

    ####
    def _getFromDatastore(self, urn):
        client = self.datastore
        # pull the repo from the urn
        identifier = utils.parts_from_urn(urn)
        return client.getJsonLD(self.bucket, identifier.id, identifier.repo)

    def _getFromGraphstore(self, urn):
        return get_rdfgraph(urn, self.graphendpoint)

    #####



import logging, os, sys
from logging import Logger
def config_notebook() -> Logger:
    """Use this to get default logging behavior while in a notebook"""
    logging.basicConfig(format='%(levelname)s : %(message)s', level=os.environ.get("LOGLEVEL", "INFO"),
                        stream=sys.stdout)
    log = logging.getLogger()
    return log

def config_app() -> Logger:
    """Use this to set default logging behavior while in a console application"""
    logging.basicConfig(format='%(levelname)s : %(message)s', level=os.environ.get("LOGLEVEL", "INFO"),
                        stream=sys.stdout)
    log = logging.getLogger()
    return log

def config_web() -> Logger:
    """Use this to set default logging behavior a web applicaiton like Flask"""
    logging.basicConfig(format='%(levelname)s : %(message)s', level=os.environ.get("LOGLEVEL", "INFO"),
                        stream=sys.stdout)
    log = logging.getLogger()
    return log


import sys
import json
def inCollab():
    """ Is this notebook being run in Goolge Collab"""
    return  'google.colab' in sys.modules

def get_txtfile(fn):
    "ret str from file"
    with open(fn, "r") as f:
        return f.read()

def get_jsfile2dict(fn):
    "get jsonfile as dict"
    #s=get_txtfile(fn)
    #return json.loads(s)
    with open(fn, "r") as f:
        return json.load(f)

def put_txtfile(fn,s,wa="w"):
    "filename to dump string to"
    #with open(fn, "w") as f:
    with open(fn, "a") as f:
        return f.write(s)


import requests
import logging as log  #have some dgb prints, that will go to logs soon/but I find it slow to have to cat the small logs everytime
log.basicConfig(filename='mgraph.log', encoding='utf-8', level=log.DEBUG,
                format='%(asctime)s.%(msecs)03d %(levelname)s %(module)s - %(funcName)s: %(message)s', datefmt='%Y-%m-%d %H:%M:%S')

"""
Goal of manage graph is to allow for the creation deletion of namespaces, and the insertion of data
This is not a class to handle querying.
"""
class ManageGraph: #really a manage graph namespace, bc a graph has several of them, &this represents only one
    """ Abstract class for managing a Graph Store
     instance for a single namespace
    """
    baseurl = "http://localhost:3030" # basically fuskei
    namespace = "temp_summary"
    path = "namespace"
    sparql = "/sparql" # blazegraph uses sparql after namespace, but let's not assume this

    def __init__(self, graphurl: str, namespace :str) :
        """initialize a class for a namespace"""
        self.baseurl = graphurl
        self.namespace = namespace
    def graphFromEndpoint(endpoint: str) -> str:
        paths = endpoint.split('/')
        paths = paths[0:len(paths) -3]
        newurl = '/'.join(paths)
        return newurl
    def createNamespace(self, quads=True):
        """ create a new namespace"""
        pass

    def deleteNamespace(self):
        """delete a namespace"""
        pass

"""
Goal of manage graph is to allow for the creation deletion of namespaces, and the insertion of data
for a BLAZEGRAPH INSTANCE
This is not a class to handle querying.
"""
class ManageBlazegraph(ManageGraph):
    """ Manages a blazegraph instance for a single namespace
    implements needed portions of the blazegraph rest API"""

    createTemplateQuad ="""com.bigdata.namespace.fffff.spo.com.bigdata.btree.BTree.branchingFactor=1024
com.bigdata.rdf.store.AbstractTripleStore.textIndex=true
com.bigdata.namespace.fffff.lex.com.bigdata.btree.BTree.branchingFactor=400
com.bigdata.rdf.store.AbstractTripleStore.axiomsClass=com.bigdata.rdf.axioms.NoAxioms
com.bigdata.rdf.sail.isolatableIndices=false
com.bigdata.rdf.sail.truthMaintenance=false
com.bigdata.rdf.store.AbstractTripleStore.justify=false
com.bigdata.rdf.store.AbstractTripleStore.quads=true
com.bigdata.journal.Journal.groupCommit=false
com.bigdata.rdf.store.AbstractTripleStore.geoSpatial=false
com.bigdata.rdf.store.AbstractTripleStore.statementIdentifiers=false
"""

    createTemplateTriples = """com.bigdata.namespace.fffff.spo.com.bigdata.btree.BTree.branchingFactor=1024
com.bigdata.rdf.store.AbstractTripleStore.textIndex=true
com.bigdata.namespace.fffff.lex.com.bigdata.btree.BTree.branchingFactor=400
com.bigdata.rdf.store.AbstractTripleStore.axiomsClass=com.bigdata.rdf.axioms.NoAxioms
com.bigdata.rdf.sail.isolatableIndices=false
com.bigdata.rdf.sail.truthMaintenance=false
com.bigdata.rdf.store.AbstractTripleStore.justify=false
com.bigdata.rdf.sail.namespace=fffff
com.bigdata.rdf.store.AbstractTripleStore.quads=false
com.bigdata.journal.Journal.groupCommit=false
com.bigdata.rdf.store.AbstractTripleStore.geoSpatial=false
com.bigdata.rdf.store.AbstractTripleStore.statementIdentifiers=false
"""
    #init w/namespace

    def createNamespace(self, quads=True):
        """ Creates a new namespace"""
        # POST / bigdata / namespace
        # ...
        # Content - Type
        # ...
        # BODY
       # add this to the createTemplates
        # # com.bigdata.rdf.sail.namespace = {namespace}
        if quads:
            template = self.createTemplateQuad
        else:
            template = self.createTemplateTriples
        template = template + f"com.bigdata.rdf.sail.namespace = {self.namespace}\n"
        url = f"{self.baseurl}/namespace"
        headers = {"Content-Type": "text/plain"}
        r = requests.post(url,data=template, headers=headers)
        if r.status_code==201:
            return "Created"
        elif  r.status_code==409:
            return "Exists"
        else:
            raise Exception(f"Create Failed. Status code: {r.status_code} {r.reason}")


    def deleteNamespace(self):
        """ deletes a blazegraph namespace"""
        # DELETE /bigdata/namespace/NAMESPACE
        url = f"{self.baseurl}/namespace/{self.namespace}"
        headers = {"Content-Type": "text/plain"}
        r = requests.delete(url, headers=headers)
        if r.status_code == 200:
            return "Deleted"
        else:
            raise Exception("Delete Failed.")


    def insert(self, data, content_type="text/x-nquads"):
        """inserts data into a blazegraph namespace"""
        # rdf datatypes: https://github.com/blazegraph/database/wiki/REST_API#rdf-data
        # insert: https://github.com/blazegraph/database/wiki/REST_API#insert
       #url = f"{self.baseurl}/namespace/{self.namespace}{self.sparql}"
       #could call insure final slash
        url = f"{self.baseurl}/namespace/{self.namespace}/{self.sparql}"
        log.info(f'insert to {url} ')
        headers = {"Content-Type": f"{content_type}"}
        r = requests.post(url,data=data, headers=headers)
        log.debug(f' status:{r.status_code}') #status:404
        log.info(f' status:{r.status_code}') #status:404
        if r.status_code == 200:
            # '<?xml version="1.0"?><data modified="0" milliseconds="7"/>'
            if 'data modified="0"'  in r.text:
                raise Exception("No Data Added: " + r.text)
            return True
        else:
            return False

    #have upload methods here
    #have graph instance:<manageGraph.ManageBlazegraph object at ..>, for url:https://graph.geocodes.ncsa.illinois.edu/blazegraph
    #tmp_endpoint=f'https://graph.geocodes.ncsa.illinois.edu/blazegraph/namespace/{repo}/sparql'

    def upload_file(self, filename, content_type="text/x-nquads"):
        "to temp namespace or final one if given"
        log.debug(f'upload_file:{filename}')
        log.info(f'upload_file:{filename}')
        #open file and insert data
        data = open(filename, 'rb').read()
        log.debug(f'insert:{filename}')
        log.info(f'insert:{filename}')
        self.insert(data, content_type)

    def upload_nq_file(self, fn=None):
        "will default to ns.nq"
        if fn:
            filename=fn
        else:
            filename=self.namespace + ".nq"
        self.upload_file(filename)

    def upload_ttl_file(self, fn=None):
        "will default to ns.ttl"
        if fn: #will want to upload ns=repo.ttl to ns=summary in the end
            filename=fn
        else:
            filename=self.namespace + ".ttl"
        self.upload_file(filename, 'Content-Type:text/x-turtle')




#!  python3
import argparse
from io import StringIO, BytesIO
import logging
import os
import sys

from ec.graph.sparql_query import queryWithSparql
from ec.reporting.report import  generateGraphReportsRepo, reportTypes, generateGraphReportsRelease
from ec.datastore import s3
from ec.logger import config_app

log = config_app()

def graphStats(args):
    """query an endpoint, store results as a json file in an s3 store"""
    if args.release is not None:
        log.info(f"Using  {args.release} for graph statisitcs  ")
        if (args.detailed):
            report_json = generateGraphReportsRelease(args.source, args.release,reportList=reportTypes["repo_detailed"] )
        else:
            report_json = generateGraphReportsRelease(args.source,
                                                       args.release, reportList=reportTypes["repo"] )
    else:
        log.info(f"Querying {args.graphendpoint} for graph statisitcs  ")
    ### more work needed before detailed works
        if args.source == "all":
             # report_json = generateGraphReportsRepo("all",
             #      args.graphendpoint, reportTypes=reportTypes)

            if (args.detailed):
                report_json = generateGraphReportsRepo("all", args.graphendpoint, reportList=reportTypes["all_detailed"] )
            else:
                report_json = generateGraphReportsRepo("all",
                                                           args.graphendpoint,reportList=reportTypes["all"])
        else:
            # report_json = generateGraphReportsRepo(args.repo,
            #   args.graphendpoint,reportTypes=reportTypes)

            if (args.detailed):
                report_json = generateGraphReportsRepo(args.source, args.graphendpoint,reportList=reportTypes["repo_detailed"] )
            else:
                report_json = generateGraphReportsRepo(args.source,
                                                           args.graphendpoint, reportList=reportTypes["repo"] )

    #data = f.getvalue()

    if (args.output):  # just append the json files to one filem, for now.
        logging.info(f" report for {args.source} appended to file")
        args.output.write(report_json)
    if not args.no_upload:
        s3Minio = s3.MinioDatastore(args.s3server, None)
        bucketname, objectname = s3Minio.putReportFile(args.s3bucket,args.source,"graph_stats.json",report_json)
    return 0
def start():
    """
        Run the generate_repo_stats program.
        query an endpoint, store results as a json file in an s3 store.
        Arguments:
            args: Arguments passed from the command line.
        Returns:
            An exit code.

    """
    parser = argparse.ArgumentParser()
    parser.add_argument('--release', dest='release',
                        help='run over release file: try: https://oss.geocodes-aws.earthcube.org/earthcube/graphs/latest/iris_release.nq'

                        )

    parser.add_argument('--graphendpoint', dest='graphendpoint',
                        help='graph endpoint' ,default="https://graph.geocodes-dev.earthcube.org/blazegraph/namespace/earthcube/")
    parser.add_argument('--s3', dest='s3server',
                        help='s3 server address (localhost:9000)', default='localhost:9000')
    parser.add_argument('--s3bucket', dest='s3bucket',
                        help='s3 server address (localhost:9000)', default='gleaner')
    parser.add_argument('--source', dest='source',
                        help='repository', default='all')

    parser.add_argument("--detailed",action='store_true',
                        dest="detailed" ,help='run the detailed version of the reports', default=False)
    parser.add_argument('--no-upload', dest = 'no_upload',action='store_true', default=False,
                        help = 'do not upload to s3 bucket ')
    parser.add_argument('--output',  type=argparse.FileType('w'), dest="output", help="dump to file")

    args = parser.parse_args()

    exitcode = graphStats(args)
    exit(exitcode)

if __name__ == '__main__':
    start()




# https://github.com/ResearchObject/bagit-ro
# This GitHub repository explains by example a profile for a BagIt bag to also be a Research Object.


# hydroshare uses bagit.

# if we need to 'package' data this may be a good form




import functools
import click
import logging
import json
import sys
from pydash.collections import find
from pydash import is_empty
from ec.gleanerio.gleaner import getSitemapSourcesFromGleaner, getGleaner
from ec.reporting.report import generateGraphReportsRepo, reportTypes, missingReport, generateIdentifierRepo, \
    generateReportStats
from ec.datastore import s3
from ec.logger import config_app
from ec.sitemap import Sitemap

log = config_app()
class EcConfig(object):
    """ Parameters that might be common to commands"""
    def __init__(self, cfgfile=None, s3server=None, s3bucket=None, graphendpoint=None, upload=None, output=None,debug=False):
        if cfgfile:
            s3endpoint, bucket, glnr = getGleaner(cfgfile)
            minio = glnr.get("minio")
            # passed paramters override the config parameters
            self.s3server = s3server if s3server else s3endpoint
            self.bucket = s3bucket if s3bucket else bucket
        else:
            self.s3server = s3server
            self.bucket = s3bucket
        self.graphendpoint = graphendpoint
        self.output = output
        self.upload = upload
        self.debug = debug

    # lets put checks as methods in here.
    # that way some checks if we can connect can be done in one place
    def hasS3(self) -> bool:
         if   ( is_empty(self.s3server) or is_empty(self.bucket) ):
             log.fatal(f" must provide a gleaner config or (s3endpoint and s3bucket)]")
             raise Exception("must provide a gleaner config or (s3endpoint and s3bucket)]")
         return True
    def hasS3Upload(self) -> bool:
         if  not self.upload and ( is_empty(self.s3server) or is_empty(self.bucket) ):
             log.fatal(f" must provide a gleaner config or (s3endpoint and s3bucket)]")
             raise Exception("must provide a gleaner config or (s3endpoint and s3bucket)]")
         return True
    def hasGraphendpoint(self, option:bool=False, message="must provide graphendpoint") -> bool:
         """ if option is not true, so if summon only, then empty is graphendpoint is ok
            """
         if    not option and is_empty(self.graphendpoint) :
             log.fatal(message)
             raise Exception(message)
         return True

def common_params(func):
    @click.option('--cfgfile', help='gleaner config file', type=click.Path(exists=True))
    @click.option('--s3server', help='s3 server address')
    @click.option('--s3bucket', help='s3 bucket')
    @click.option('--graphendpoint', help='graph endpoint')
    @click.option('--upload/--no-upload', help='upload to s3 bucket', default=True)
    @click.option('--output', help='dump to file', type=click.File('wb'))
    @click.option('--debug/--no-debug', default=False, envvar='REPO_DEBUG')
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        return func(*args, **kwargs)
    return wrapper

@click.group()
@common_params
def cli( cfgfile,s3server, s3bucket, graphendpoint, upload, output, debug):
   obj = EcConfig(cfgfile,s3server, s3bucket, graphendpoint, upload, output, debug)

@cli.command()

@click.option('--source', help='One or more repositories (--source a --source b)', multiple=True)
@click.option('--milled/--no-milled', help='include milled', default=False)
@click.option('--summononly', help='check summon only', is_flag=True, default=False)
@common_params
def missing_report(cfgfile,s3server, s3bucket, graphendpoint, upload, output, debug, source, milled, summononly):
    # name missing-report
    ctx = EcConfig(cfgfile, s3server, s3bucket, graphendpoint, upload, output, debug)
    output = ctx.output
    upload = ctx.upload
    bucket = ctx.bucket
    s3server = ctx.s3server
    graphendpoint = ctx.graphendpoint  # not in gleaner file, at presen
    ctx.hasS3()
    ctx.hasGraphendpoint(option=summononly, message="must provide graphendpoint if you are checking the graph" )

    log.info(f"s3server: {s3server} bucket:{bucket} graph:{graphendpoint}")
    s3Minio = s3.MinioDatastore(s3server, {})
    sources = getSitemapSourcesFromGleaner(cfgfile)
    sources = list(filter(lambda source: source.get('active'), sources))
    sources_to_run = source  # optional if null, run all

    for i in sources:
        source_url = i.get('url')
        source_name = i.get('name')
        if sources_to_run is not None and len(sources_to_run) >0:
            if not find (sources_to_run, lambda x: x == source_name ):
                continue
        sm = Sitemap(source_url)
        if not sm.validUrl():
            log.error("Invalid or unreachable URL: {source_url} ")
            break
        try:

            report = missingReport(source_url, bucket, source_name, s3Minio, graphendpoint, milled=milled, summon=summononly)
            report = json.dumps(report,  indent=2)
            if output:  # just append the json files to one filem, for now.
                log.info(f"report for {source_name} appended to file")
                output.write(bytes(report, 'utf-8'))
            if upload:
                s3Minio.putReportFile(bucket, source_name, "missing_report.json", report)
        except Exception as e:
            log.error(f"could not write missing report for {source_name} to s3server:{s3server}:{bucket} error:{str(e)}")
    return

@cli.command()
@click.option('--source', help='One or more repositories (--source a --source b)', multiple=True)
@click.option('--detailed', help='run the detailed version of the reports',is_flag=True, default=False)
@common_params
def graph_stats(cfgfile,s3server, s3bucket, graphendpoint, upload, output, debug, source, detailed):
    ctx = EcConfig(cfgfile, s3server, s3bucket, graphendpoint, upload, output, debug)
    output = ctx.output
    no_upload = ctx.upload
    graphendpoint = ctx.graphendpoint
    s3server = ctx.s3server
    s3bucket = ctx.bucket

    ctx.hasS3Upload()
    ctx.hasGraphendpoint(message=" must provide graphendpoint")
    if upload:
        s3Minio = s3.MinioDatastore(s3server, None)
    """query an endpoint, store results as a json file in an s3 store"""
    log.info(f"Querying {graphendpoint} for graph statisitcs  ")
    if source:
        sources = source
    else:
        sources = getSitemapSourcesFromGleaner(cfgfile)
        sources = list(filter(lambda source: source.get('active'), sources))
        sources = list(map(lambda r: r.get('name'), sources))
### more work needed before detailed works
    if "all" in source:
        if detailed:
            report_json = generateGraphReportsRepo("all", graphendpoint, reportList=reportTypes["all_detailed"])
        else:
            report_json = generateGraphReportsRepo("all", graphendpoint,reportList=reportTypes["all"])
        if output:  # just append the json files to one filem, for now.
            log.info(f" report for ALL appended to file")
            output.write(report_json)
        if upload:
            bucketname, objectname = s3Minio.putReportFile(s3bucket, "all", "graph_report.json", report_json)
    else:
        for s in sources:
            if detailed:
                report_json = generateGraphReportsRepo(s, graphendpoint, reportList=reportTypes["repo_detailed"])
            else:
                report_json = generateGraphReportsRepo(s, graphendpoint, reportList=reportTypes["repo"])
            if output:  # just append the json files to one filem, for now.
                log.info(f" report for {s} appended to file")
                output.write(bytes(report_json, 'utf-8'))
            if upload:
                bucketname, objectname = s3Minio.putReportFile(s3bucket, s, "graph_report.json", report_json)
    return

@cli.command()
@click.option('--source', help='One or more repositories (--source a --source b)', multiple=True)
@click.option('--json', help='output json format', is_flag=True, default=True)
@common_params
def identifier_stats(cfgfile,s3server, s3bucket, graphendpoint, upload, output, debug, source, json):
    filename = 'identifier_metadata_summary'
    if cfgfile:
        s3endpoint,  bucket, glnr=getGleaner(cfgfile)
        minio = glnr.get("minio")
        # passed paramters override the config parameters
        s3server = s3server if s3server else s3endpoint
        bucket = s3bucket if s3bucket else bucket
    else:
        s3server = s3server
        bucket = s3bucket

    if is_empty(s3server) or is_empty(bucket):
        logging.fatal(f" must provide a gleaner config or (s3endpoint and s3bucket)]")
        raise Exception(" must provide a gleaner config or (s3endpoint and s3bucket)]")

## output is file
    if json:
        filename = filename + '.json'
    else:
        filename = filename + '.csv'

    logging.info(f" s3server: {s3server} bucket:{bucket}")

    s3Minio = s3.MinioDatastore(s3server, None)
    if source:
        sources = source
    else:
        sources = getSitemapSourcesFromGleaner(cfgfile)
        sources = list(filter(lambda source: source.get('active'), sources))
        sources = list(map(lambda r: r.get('name'), sources))
    for repo in sources:
        try:
            identifier_stats = generateIdentifierRepo(repo, bucket, s3Minio)
            if json:
                o = identifier_stats.to_json(orient='records', indent=2)
            else:
                o = identifier_stats.to_csv(index=False)

            if output:
                logging.info(f" report for {repo} appended to file")
                output.write(bytes(o, 'utf-8'))
            if upload:
                s3Minio.putReportFile(bucket, repo, filename, o)

        except Exception as e:
            logging.info('Missing keys: ', e)
    return

@cli.command()
@click.option('--url', help='URL of the source CSV file', required=True)
@click.option('--community', help='Community', required=False)
@click.option('--graphendpoint', help='Graph endpoint of summary', required=True)
@common_params
def generate_report_stats(cfgfile, s3server, s3bucket, graphendpoint, upload, output, debug, url, community):
    ctx = EcConfig(cfgfile, s3server, s3bucket, graphendpoint, upload, output, debug)
    upload = ctx.upload
    bucket = ctx.bucket
    s3server = ctx.s3server
    graphendpoint = ctx.graphendpoint  # not in gleaner file
    ctx.hasS3()
    ctx.hasGraphendpoint(message=" must provide graphendpoint")

    log.info(f"s3server: {s3server} bucket:{bucket} graph:{graphendpoint}")
    s3Minio = s3.MinioDatastore(s3server, {})

    if is_empty(community):
        community = 'all'
    # graphendpoint needs to be summary
    report = generateReportStats(url, bucket, s3Minio, graphendpoint, community)

    if upload:
        s3Minio.putReportFile(bucket, "tenant", f"report_{community}_stats.json", report)
    return


# @cli.command()
# # @click.option('--cfgfile', help='gleaner config file', default='gleaner', type=click.Path(exists=True))
# # no default for s3 parameters here. read from gleaner. if provided, these override the gleaner config
# # @click.option('--s3server', help='s3 server address')
# # @click.option('--s3bucket', help='s3 bucket')
# @click.option('--source', help='One or more repositories (--source a --source b)', multiple=True)
# @click.option('--json', help='output json format', is_flag=True, default=True)
# @click.option('--detailed', help='run the detailed version of the reports',is_flag=True, default=False)
# @click.option('--milled/--no-milled', help='include milled', default=False)
# @click.option('--summononly', help='check summon only',is_flag=True, default=False)
# # @click.pass_obj
# @common_params
# @click.pass_context
# def run_all (ctx, cfgfile,s3server, s3bucket, graphendpoint, upload, output, debug, source, json,detailed, milled, summononly ):
# # this probably needs to run to a try catch block where ratehr than method doing a sys.exit, they
# # toss an exception, so if one report fails, the others run.
#     try:
#         ctx.forward(missing_report)
#         # ctx.invoke( missing_report, cfgfile=cfgfile,s3server=s3server,
#         #             s3bucket=s3bucket,
#         #             graphendpoint=graphendpoint,
#         #             upload=upload, output=output,
#         #             debug=debug,
#         #             source=source,
#         #             milled=milled,
#         #             summononly=summononly
#         #             )
#     except Exception as e:
#         log.error("missing report failed ")
#     try:
#         identifier_stats(cfgfile, s3server, s3bucket, graphendpoint, upload, output, debug, source, json)
#     except Exception as e:
#         log.error("identifier stats failed")
#     try:
#
#         graph_stats(cfgfile, s3server, s3bucket, graphendpoint, upload, output, debug, source, detailed)
#     except Exception as e:
#         log.error("graph stats failed")

if __name__ == '__main__':
    cli()
    sys.exit(0)



from ec.objects.ecobjectmanager import EcObjectManager

def get_from_geocodes(urn, endpoint="https://graph.geocodes-dev.earthcube.org/blazegraph/namespace/earthcube/sparql"):
    """get an object from the graph, and return as a JSON-LD file """
    manager = EcObjectManager(endpoint, None, None)
    return manager.getFromStore(urn, source='graph', form='jsonld')


import csv
import re
from urllib import request
import click
import requests
from ec.logger import config_app
from ec.datastore import s3

log = config_app()

# Example of CSV URL: 'https://docs.google.com/spreadsheets/d/1pqZpMWqQFwUrleHXPbvXqXX59Xcj1Yrtqt2nJTh1reM/pub?output=csv'
def read_community_items_CSV(gsheet_csv_url):
    response = request.urlopen(gsheet_csv_url)
    csv_reader = csv.DictReader(response.read().decode('utf-8').splitlines())

    data_list = [
        row for row in csv_reader
        if row.get('Type') == 'Dataset'
    ]
    return data_list


def check_url_for_jsonld(url):
    try:
        # Fetch the content of the URL
        response = requests.get(url)
        response.raise_for_status()  # Raise an error for bad status codes

        # Search for <script> tags with type="application/ld+json" using a regular expression
        pattern = r'<script[^>]+type="application/ld\+json"[^>]*>(.*?)</script>'
        matches = re.findall(pattern, response.text, re.DOTALL)

        if matches:
            # Return True if JSON-LD data is found
            return True
        else:
            # Return False if no JSON-LD data is found
            return False
    except requests.RequestException as e:
        print(f"An error occurred while trying to fetch the URL: {e}")
        return False

def generate_sitemap(gsheet_csv_url):
    data_list = read_community_items_CSV(gsheet_csv_url)

    sitemap_entries = []
    for data in data_list:
        url = data['Dataset Webpage URL']

        if check_url_for_jsonld(url):
            entry = f"""
    <url>
        <loc>{url}</loc>
    </url>"""
            sitemap_entries.append(entry)
        else:
            print(f"No JSON-LD data found for {url}.")

    sitemap_xml = f"""<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">
{''.join(sitemap_entries)}
</urlset>"""

    return sitemap_xml

@click.command()
@click.option('--url_items', help='URL for Community Items of the source CSV file', required=True)
@click.option('--s3server', help='s3 server address')
@click.option('--s3bucket', help='s3 bucket')
def convert_gsheet_csv_to_sitemap(url_items, s3server, s3bucket):
    s3Minio = s3.MinioDatastore(s3server, None)
    sitemap = generate_sitemap(url_items)
    # upload the generated sitemap to s3 bucket
    s3Minio.putSitemapFile(s3bucket, "geochemistry_sitemap.xml", sitemap)
    return sitemap

def start():
    """
        Read datasets from the google sheet and convert them to a sitemap
        Arguments:
            args: Arguments passed from the command line.
        Returns:
            a sitemap

    """
    result = convert_gsheet_csv_to_sitemap()

if __name__ == '__main__':

    result = start()




import argparse
import logging
import sys
from pydash import is_empty
from ec.gleanerio.gleaner import getSitemapSourcesFromGleaner, getGleaner
from ec.datastore import s3
from ec.reporting.report import generateIdentifierRepo

def summarizeIdentifierMetadata(args):
    if (args.cfgfile):
        s3endpoint,  bucket, glnr= getGleaner(args.cfgfile)
        minio = glnr.get("minio")
        # passed paramters override the config parameters
        s3server = args.s3server if args.s3server else s3endpoint
        bucket = args.s3bucket if args.s3bucket else bucket
    else:
        s3server = args.s3server
        bucket = args.s3bucket

    if is_empty(s3server) or is_empty(bucket):
        logging.fatal(f" must provide a gleaner config or (s3endpoint and s3bucket)]")
        return 1

    if args.json:
        filename = args.output.rsplit(".", 1)[0] + '.json'
    else:
        filename = args.output.rsplit(".", 1)[0] + '.csv'
    if args.output:
        output_file = open(filename, 'w')

    logging.info(f" s3server: {s3server} bucket:{bucket}")

    s3Minio = s3.MinioDatastore(s3server, None)
    if args.source:
        sources = args.source
    else:
        sources = getSitemapSourcesFromGleaner(args.cfgfile)
        sources = list(filter(lambda source: source.get('active'), sources))
        sources = list(map(lambda r: r.get('name'), sources))
    for repo in sources:
        try:
            identifier_stats = generateIdentifierRepo(repo, bucket, s3Minio)
            if args.json:
                o = identifier_stats.to_json(orient='records', indent=2)
            else:
                o = identifier_stats.to_csv(index=False)

            if args.output:
                logging.info(f" report for {repo} appended to file")
                output_file.write(o)
            if not args.no_upload:
                s3Minio.putReportFile(bucket, repo, filename, o)

        except Exception as e:
            logging.info('Missing keys: ', e)
    return 0

def start():
    """
        Run the summarize_identifier_metadata program.
        Get a list of active repositories from the gleaner file.
        For each repository, summarize identifierTypes and matchedPath, then write these information to a file and upload it to s3.
        Arguments:
            args: Arguments passed from the command line.
        Returns:
            An exit code.
    """
    parser = argparse.ArgumentParser()
    # source of sources, and default s3 store.
    #   at present, graph endpoint is no longer in gleaner
    parser.add_argument('--cfgfile', dest='cfgfile',
                        help='gleaner config file')
    # no default for s3 parameters here. read from gleaner. if provided, these override the gleaner config
    parser.add_argument('--s3', dest='s3server',
                        help='s3 server address ')
    parser.add_argument('--s3bucket', dest='s3bucket',
                        help='s3 bucket ')
    parser.add_argument('--no_upload', dest='no_upload', action='store_true', default=False,
                        help='do not upload to s3 bucket ')
    parser.add_argument('--output', default='identifier_metadata_summary.csv',
                        dest='output', help='dump to file')
    parser.add_argument('--json', dest='json', action='store_false', default=True,
                        help='output json format')
    parser.add_argument('--source', action='append', help="one or more repositories (--source a --source b)")

    args = parser.parse_args()
    exitcode = summarizeIdentifierMetadata(args)
    sys.exit(exitcode)

if __name__ == '__main__':
    start()



import json
import logging
from typing import Union, Tuple, Any

from jsonschema import validate
from jsonschema.exceptions import ValidationError
from string import Template

try:
    import importlib.resources as pkg_resources
except ImportError:
    # Try backported to PY<37 `importlib_resources`.
    import importlib_resources as pkg_resources

from pydash import ends_with, sort
from pyld import jsonld
import ec.sos_json.schemas as schemafiles

jsonld_context = context = {"@vocab": "https://schema.org/"}


def _getSchemaFromResources(filename) :
    """ retrieves json schema file from the  ec.sos_json.schemas directory when in a package"""
    resourcename = f"{filename}.json"
    resource = pkg_resources.read_text(schemafiles, resourcename)
    schema = json.loads(resource)
    return schema


def isValidJSON(jsonData: str)-> bool:
    """Is Json Valid"""
    try:
        json.loads(jsonData)
    except ValueError as err:
        return False
    return True

def listSchemaFilesFromResources() -> str:
    """ retrieves schema file from the schemas folder when in a package"""
    resource = pkg_resources.contents(schemafiles)
    files = filter( lambda f: ends_with(f, ".json"), resource)
    files = sort(list(files))
    return files
def validateJson2Schema(json_data: Any, schemaname='GeoCodes-DatasetSchema') -> Tuple[bool,str]:
    """REF: https://json-schema.org/

    Parameters:
        json_data: JSON Object (aka json.loads())
        schemaname: name of schem in ec.sos_json.shcemas: (GeoCodes-DatasetSchema or GeoCodes-ECRR-DatasetSchema.json)

    Returns:
        bool: is valid
        message: "Given JSON data is Valid" or "Given JSON data is InValid"
        """
    # Describe what kind of json you expect.
    execute_api_schema = _getSchemaFromResources(schemaname)

    try:
        validate(instance=json_data, schema=execute_api_schema)
    except ValidationError as err:
        logging.error(err)
        err = f"Given JSON data is InValid {err}"
        return False, err

    message = "Given JSON data is Valid"
    return True, message


def validateSosDataset(jsonData: str) -> bool:
    """ validates a SOS Dataset"""
    try:
        json_data = json.loads(jsonData)
    except ValueError as err:
        return False
    valid2_schema, err = validateJson2Schema(jsonData)
    return valid2_schema


def validateEcrrTool(jsonData : str) -> bool:
    """
       validated
        Parameters:
            jsonData: jsonld string returns the string
            form: 'compact'--compact form,
                   'frome'--framed usngin schemaType,
                   'jsonld' - passed form
            schemaType: type of the schema default="Dataset"
        Returns:
            bool.

    """
    try:
        json_data = json.loads(jsonData)
    except ValueError as err:
        return False
    valid2_schema, err = validateJson2Schema(jsonData, schemaname="GeoCodes-ECRR-DatasetSchema.json")
    return valid2_schema


def compact_jld_str(jld_str: str) -> str:
    """
       depreciated use formatted_jsonld

    """
    doc = json.loads(jld_str)
    compacted = jsonld.compact(doc, jsonld_context)
    r = json.dumps(compacted, indent=2)
    return r


def formatted_jsonld(jld_str: str, form="compact", schemaType="Dataset") -> str:
    """
       returns a formatted JSONLD string based on the input
        Parameters:
            jld_str: jsonld string returns the string
            form: 'compact'--compact form,
                   'frome'--framed usngin schemaType,
                   'jsonld' - passed form
            schemaType: type of the schema default="Dataset"
        Returns:
            string.

    """
    if (form == 'jsonld'):
        return jld_str

    elif (form == "frame"):
        frame = (' {\n'
                 '              "@context": {\n'
                 '                "@vocab": "https://schema.org/",\n'
                 '                    "rdf": "http://www.w3.org/1999/02/22-rdf-syntax-ns#",\n'
                 '                    "rdfs": "http://www.w3.org/2000/01/rdf-schema#",\n'
                 '                    "schema": "https://schema.org/",\n'
                 '                    "xsd": "http://www.w3.org/2001/XMLSchema#"\n'
                 '              },\n'
                 '              "@type": "schema:${schemaType}"\n'
                 '  }\n '
                 )
        f_template = Template(frame)
        thsGraphQuery = f_template.substitute(schemaType=schemaType)

        frame_doc = json.loads(thsGraphQuery)
        doc = json.loads(jld_str)
 #       doc = jsonld.flatten(doc)
        # issue where @id on Dataset and @id on Dataset.identifer were the same. If so
        # @type = ['Dataset', 'PropertyValue'] was the conversion.

        framed = jsonld.frame(doc, frame_doc)

        r = json.dumps(framed, indent=2)
        return r
    else:  # compact
        doc = json.loads(jld_str)
        compacted = jsonld.compact(doc, jsonld_context)
        r = json.dumps(compacted, indent=2)
        return r



import copy
import io
import json
import os
import shutil
import subprocess
from typing import Tuple, Any

import yaml


def endpointUpdateNamespace( fullendpoint: str, namepsace='temp') -> str:
    paths = fullendpoint.split('/')
    paths[len(paths)-2] = namepsace
    newurl= '/'.join(paths)
    return newurl

def getNabu( cfgfile: str) -> Tuple[str, object]:
    with open(cfgfile, "r") as stream:
        try:
            cfg =(yaml.safe_load(stream))
        except yaml.YAMLError as exc:
            print(exc)
    endpoint = cfg['sparql']['endpoint']
    return endpoint, cfg


def getNabuFromFile( cfgfile_obj) -> Tuple[str, object]:
    """ read a file from argparse
    when reading args where a file is returned, use this one


    Parameters:
        cfgfile_obj: an argparse.FileType, maybe an io.FileIo
"""

    """ when reading args where a file is returned, use this one"""
    cfg = yaml.safe_load(cfgfile_obj)
    endpoint = cfg['sparql']['endpoint']
    #     endpoint = cfg['sparql']['endpoint']
    return endpoint, cfg

def reviseNabuConfGraph(cfg, endpoint) -> object:
    newcfg = copy.deepcopy(cfg)
    newcfg['sparql']['endpoint'] = endpoint
    return newcfg
def reviseNabuConfS3(cfg,  bucket) -> object:
    newcfg = copy.deepcopy(cfg)
    newcfg['s3']['bucket'] = bucket
    return newcfg

def runNabu(cfg, repo,glcon="~/indexing/glcon"):
    if shutil.which(glcon) is not None:
        filename = f"nabu_{repo}" # avoid possible naming conflicts
        with open(filename, 'w') as f:
            yaml.dump(cfg, f)
        executeNabu = f"{glcon} nabu prefix --cfg {filename} --prefix summoned/{repo}"
        fullpath = os.path.abspath(filename)
        try:
            result = subprocess.run([glcon, "nabu", "prefix",
                                     "--cfg", fullpath,
                                     "--prefix", f"summoned/{repo}",
                                     ],
                                    capture_output=True,
                                    )
            if result.returncode != 0:
                raise Exception(f"return code:{result.returncode} try from command line: '{glcon}  nabu prefix --cfg {filename} --prefix summoned/{repo}'  {result}")
        except Exception as ex:
            raise Exception(f"failed  try from command line:  '{glcon}  nabu prefix --cfg {filename} --prefix summoned/{repo}'  reason{ex}")
        # delete config file here
        return result.stdout
    else:
        raise Exception(f"glcon not found at {glcon}. Pass path to glcon with --glcon")



def getGleaner( cfgfile) -> Tuple[str,str,Any]:
    with open(cfgfile, "r") as stream:
        try:
            cfg =(yaml.safe_load(stream))
        except yaml.YAMLError as exc:
            print(exc)
    bucket = cfg['minio']['bucket']
    s3endpoint = cfg['minio']['address']
    return s3endpoint,  bucket, cfg
def getGleanerFromFile( cfgfile_obj):
    """ read a file from argparse
    when reading args where a file is returned, use this one


    Parameters:
        cfgfile_obj: an argparse.FileType, maybe an io.FileIo
    """
    cfg = yaml.safe_load(cfgfile_obj)
    bucket = cfg['minio']['bucket']
    s3endpoint = cfg['minio']['address']
    return s3endpoint,  bucket, cfg

def getSourcesFromGleaner(cfgfile, sourcename=None):
    s3endpoint,  bucket, cfg = getGleaner(cfgfile)
    sources = cfg['sources']
    if sourcename is None:
        return sources
    else:
        matched = list(filter(lambda source: source.get('name') == sourcename, sources))
        if len(matched) == 1:
            return matched[0]
        else:
            return None


def getSitemapSourcesFromGleaner(cfgfile, sourcename=None):
    s3endpoint,  bucket, cfg = getGleaner(cfgfile)
    sources = cfg['sources']
    sources = list(filter(lambda source: source.get('sourcetype') == "sitemap", sources))
    if sourcename is None:
        return sources
    else:
        matched = list(filter(lambda source: source.get('name') == sourcename, sources))
        if len(matched) == 1:
            return matched[0]
        else:
            return None


def reviseGleanerConf(cfg: str, bucket:str) -> object:
    newcfg = copy.deepcopy(cfg)
    newcfg['minio']['bucket'] = bucket
    return newcfg

def runGleaner(cfg, repo,glcon="~/indexing/glcon"):
    if shutil.which(glcon) is not None:
        filename = f"gleaner_{repo}" # avoid possible naming conflicts
        with open(filename, 'w') as f:
            yaml.dump(cfg, f)

        executeApp = f"{glcon} gleaner batch --cfg {filename} --source {repo}"
        fullpath = os.path.abspath(filename)
        try:
            result = subprocess.run([glcon, "gleaner", "batch",
                                     "--cfg", fullpath,
                                     "--source", repo,
                                     ],
                                    capture_output=True,
                                    )
            if result.returncode != 0:
                raise Exception(f"return code:{result.returncode} try from command line: '{glcon}  gleaner batch --cfg {filename} --source {repo}'  {result}")
        except Exception as ex:
            raise Exception(f"failed  try from command line:  '{glcon}  gleaner batch --cfg {filename} --source {repo}'  reason{ex}")
        # delete config file here
        return result.stdout
    else:
        raise Exception(f"glcon not found at {glcon}. Pass path to glcon with --glcon")

## this takes a string with the jsonld, and
def runIdentifier( jsonld_str,glncfg="../resources/configs/geocodedemo/gleaner", glcon="~/indexing/glcon"):
    isvalid, err = _validateJSON(jsonld_str)
    if not isvalid:
         raise Exception( f"invalid json:  {err}" )
    if shutil.which(glcon) is not None:
    #     filename = f"./gleaner_id" # avoid possible naming conflicts
    #     with open(filename, 'w') as f:
    #         yaml.dump(cfg, f)
    #     executeApp = f"{glcon}  tools id --idtype identifiersha "
        try:
            result = subprocess.run([glcon,  "tools", "id",
                                     "--idtype", "identifiersha",
                                    "--cfg", glncfg
                                     ],
                                    capture_output=True,
                                     input=bytes(jsonld_str, 'utf-8')
                                    )
            if result.returncode != 0:
                raise Exception(f"return code:{result.returncode} try from command line: '{glcon} tools id --idtype identifiersha'  {result}")
        except Exception as ex:
            raise Exception(f"failed  try from command line:  '{glcon} tools id --idtype identifiersha'  reason{ex}")
        # delete config file here
        return result.stdout
    else:
        raise Exception(f"glcon not found at {glcon}. Pass path to glcon with --glcon")

## needs to be in utils
def _validateJSON(jsonData):
    try:
        json.loads(jsonData)
    except ValueError as err:
        return False, err
    return True, None



import logging
from io import StringIO
from string import Template
from typing import Union

import pandas
import sparqldataframe
from rdflib import URIRef, BNode, Literal, Graph,Namespace, RDF
import rdflib
import json

from  ec.graph.sparql_query import queryWithSparql

HTTPS_SCHEMA_ORG = "https://schema.org/"
HTTP_SCHEMA_ORG = "http://schema.org/"
BASE_SHCEMA_ORG = HTTPS_SCHEMA_ORG
context = f"@prefix : <{BASE_SHCEMA_ORG}> ."

### BLAZEGRAPH
'''original fetch all from temporary namespace'''
def get_summary4repo(endpoint: str) -> pandas.DataFrame:
    logging.info("Running Summary Query to Get all records")
    df = queryWithSparql("all_summary_query", endpoint)
    return df
    # file = '../resources/sparql/summary_query.txt'
    # with open(file, 'r') as f:
    #     lines = f.read()
    # df = sparqldataframe.query(endpoint,lines)
    # return df

''' fetch all from graph namespace'''
def get_summary4graph(endpoint : str) -> pandas.DataFrame:
    logging.info("Running Summary Query to Get all records")
    df= queryWithSparql("all_summary_query",endpoint)
    return df
    # file = '../resources/sparql/all_summary_query.sparql'
    # with open(file, 'r') as f:
    #     lines = f.read()
    # df = sparqldataframe.query(endpoint,lines)
    # return df

''' fetch subset  from graph namespace'''
def get_summary4repoSubset(endpoint: str, repo : str) -> pandas.DataFrame:
    logging.info(f"Running Summary Query to Get {repo} records")
    df = queryWithSparql("repo_summary_query",endpoint, parameters={"repo":repo})
    return df
    # file = '../resources/sparql/repo_summary_query.sparql'
    # with open(file, 'r') as f:
    #     lines = f.read()
    # #query = getFileFromResources(f"{template_name}")
    # #q_template = Template(query)
    # q_template = Template(lines)
    # thsGraphQuery = q_template.substitute(repo=repo)
    #
    # df = sparqldataframe.query(endpoint,thsGraphQuery)
    # return df

###
# from dataframe

# need a from release flag for rdflib terms ...
# (rdflib.term.Variable('datep'), None) (rdflib.term.Variable('description'), rdflib.term.Literal('Global stacks of up to a million event-windowed seismograms using short-term to long-term averages (STA/LTA) in different frequency bands for vertical broadband data (1990-2012) available from the IRIS DMC.  Long period versions include vertical and horizontal component data.\r\n')) (rdflib.term.Variable('g'), rdflib.term.URIRef('urn:ec-geocodes:iris:7c61b564beb0be54aca8085c1d0a3b311ffe0cbb')) (rdflib.term.Variable('kw'), rdflib.term.Literal('seismic,seismology,geophysics,globalstacks')) (rdflib.term.Variable('name'), rdflib.term.Literal('Global stacks of millions of seismograms')) (rdflib.term.Variable('placenames'), rdflib.term.Literal('No spatialCoverage')) (rdflib.term.Variable('pubname'), None) (rdflib.term.Variable('resourceType'), rdflib.term.URIRef('https://schema.org/Dataset')) (rdflib.term.Variable('sosType'), rdflib.term.URIRef('https://schema.org/Dataset')) (rdflib.term.Variable...
####

def summaryDF2ttl(df: pandas.DataFrame, repo: str, from_release=False) -> tuple[ Union[str,bytes], Graph]:
    "summarize sparql query returns turtle string and rdf lib Graph"
    urns = {}
    def is_str(v):
        return type(v) is str
    g = Graph()
    ## ##########
    # Not officially a standard schema format.
    # we might want to use our own namespace in the future
    ###########
    g.bind("ecsummary", BASE_SHCEMA_ORG)
    ecsummary = Namespace(BASE_SHCEMA_ORG)
    sosschema = Namespace(BASE_SHCEMA_ORG)

    for index, row in df.iterrows():
        logging.debug(f'dbg:{row}')
        gu=row["g"]

        graph_subject = URIRef(gu)
        #skip the small %of dups, that even new get_summary.txt * has
        if not urns.get(gu):
            urns[gu]=1
        else:
            #print(f'already:{there},so would break loop')
            continue #from loop


        rt_=row.get('resourceType')
        rt=rt_.replace("https://schema.org/","")
        logging.debug(f'rt:{rt}')

        name=json.dumps(row.get('name')) #check for NaN/fix
        if not name:
            name=f'""'
        if not is_str(name):
            name=f'"{name}"'
        if name=="NaN": #this works, but might use NA
            name=f'"{name}"'
# description
        description=row['description']
        if is_str(description):
            sdes=json.dumps(description)
            #sdes=description.replace(' / ',' \/ ').replace('"','\"')
            #sdes=sdes.replace(' / ',' \/ ').replace('"','\"')
          # sdes=sdes.replace('"','\"')
        else:
            sdes=f'"{description}"'
# keywords
        kw_=row['kw']
        if is_str(kw_):
            kw=json.dumps(kw_)
        else:
            kw=f'"{kw_}"'
# publisher
        pubname=row['pubname']
        if   pandas.isna(pubname) or  pubname=="No Publisher":
            pubname = repo
        #if no publisher urn.split(':')
        #to use:repo in: ['urn', 'gleaner', 'summoned', 'opentopography', '58048498c7c26c7ab253519efc16df237866e8fe']
        #as of the last runs, this was being done per repo, which comes in on the CLI, so could just use that too*


# date
        datep=row['datep']
        if datep == "No datePublished":
            datep=pandas.NA
        else:
            # Truncate to year
            datep = datep.split('-')[0]
        # Query should not return "No datePublished" is not a valid Date "YYYY-MM-DD" so
        # UI Date Select failed, because it expects an actual date
        #   Empty values might be handled in the UI...,
        #   or the repository valiation reporting


        placename=row['placenames']


        ##############
        # output
        # write to f StringIO()
        # for RDF graph, using sub, verp object
        ###############
        s=row['subj']
# RDF.TYPE

#         if rt == "tool":
#             g.add((graph_subject,RDF.type, sosschema.SoftwareApplication) )
#         else:
#             g.add((graph_subject, RDF.type, sosschema.Dataset))

        # original aummary query wrote out strings, then converted back to schema uri... just skip that step
        # RDF.TYPE
        rt = row['sosType']
        g.add((graph_subject, RDF.type, URIRef(rt)))

# ecsummary.name
        if (pandas.isnull( row.get('name'))):
            g.add((graph_subject, ecsummary.name, Literal("")))
        else:
            g.add( (graph_subject, ecsummary.name, Literal( row.get('name') ) ) )

# ecsummary.description
        g.add((graph_subject, ecsummary.description, Literal(description)))

# ecsummary.keywords
        g.add((graph_subject, ecsummary.keywords, Literal(kw_)))
# ecsummary.publisher
        if pandas.notna(pubname):
            g.add((graph_subject, ecsummary.publisher, Literal(pubname)))
# ecsummary.place
        g.add((graph_subject, ecsummary.place, Literal(placename)))
# ecsummary date
        if pandas.notna(datep):
            #might be: "No datePublished" ;should change in qry, for dv's lack of checking
            # Query should not return "No datePublished" is not a valid Date "YYYY-MM-DD" so
            # UI Date Select failed, because it expects an actual date
            #   Empty values might be handled in the UI...,
            #   or the repository valiation reporting
            g.add((graph_subject, ecsummary.date, Literal(datep)))
# ecsummary subjectOf
        g.add((graph_subject, ecsummary.subjectOf, URIRef(s)))

# ecsummary.distribution
        du= row.get('url') # check now/not yet
        if is_str(du):
            g.add((graph_subject, ecsummary.distribution, URIRef(s)))
# spatial

# ecsummary.latitude
        mlat= row.get('maxlat') # check now/not yet
        if is_str(mlat):
            g.add((graph_subject, ecsummary.latitude, Literal(mlat)))
        mlon= row.get('maxlon') # check now/not yet
        if is_str(mlon):
            g.add((graph_subject, ecsummary.longitude, Literal(mlon)))

# ecsummary.encodingFormat
        encodingFormat= row.get('encodingFormat') # check now/not yet
        if is_str(encodingFormat):
            g.add((graph_subject, ecsummary.encodingFormat, Literal(encodingFormat)))
        #see abt defaults from qry or here, think dv needs date as NA or blank/check
        #old:
        #got a bad:         :subjectOf <metadata-doi:10.17882/42182> .
        #incl original subj, just in case for now
        #lat/lon not in present ui, but in earlier version

        mindepth = row['minDepth']
        maxdepth = row['maxDepth']
        if is_str(mindepth):
            g.add((graph_subject, ecsummary.minDepth, Literal(mindepth)))
        if is_str(maxdepth):
            g.add((graph_subject, ecsummary.maxDepth, Literal(maxdepth)))
        #### end for ####
    return g.serialize(format='longturtle'), g
# g is an RDF graph that can be dumped using
# output_string = g.serialize(format='longturtle')
# output_string = g.serialize(format="json-ld")
# or other formats





import json
import logging
import logging
from datetime import datetime
from io import BytesIO

import minio
import pandas
import pydash
from minio.commonconfig import CopySource, REPLACE
from pydash import is_empty
from pydash.collections import find

def shaFroms3Path(path, extension=None):
    split = path.split("/")
    sha = split[len(split)-1]
    if extension is not None:
        sha = pydash.strings.replace_end(sha, extension, '')
    return sha


"""
Basic abstraction, in case someone want to store files in a 
different method
"""
class bucketDatastore():
    endpoint = "http://localhost:9000" # basically minio
    options = {"secure":True,
               "region": 'us-west-2',
#               "access_key": None,
 #              "secret_key": None
               }
    default_bucket="gleaner"
    paths = {"report":"reports",
             "summon": "summoned",
             "milled":"milled",
             "graph":"graphs",
             "release":"graphs",
             "archive":"archive",
             "collection":"collections",
             "sitemap":"sitemaps"
    }

    def __init__(self, s3endpoint, options, default_bucket="gleaner"):
        self.endpoint = s3endpoint
        self.options = options
        self.default_bucket = default_bucket

    def listPath(self, bucket, path, include_user_meta=False):
        pass
    def countPath(self, bucket, path):
        count = len(list(self.listPath(bucket, path)))
        return count

    def DataframeFromPath(self, bucket, path, include_user_meta=False):
        pass
# who knows, we might implement on disk, or in a database. This just separates the data from the annotated metadata
    def getFileFromStore(self, s3ObjectInfo):
        pass
    def getFileMetadataFromStore(self, s3ObjectInfo):
        pass
    def putTextFileToStore(self,data, s3ObjectInfo ):
        f = BytesIO()
        length = f.write(bytes(data, 'utf-8'))
        f.seek(0)
        resp = self.s3client.put_object(s3ObjectInfo.bucket_name, s3ObjectInfo.object_name, f,length=length)
        return resp.bucket_name, resp.object_name
    def copyObject(self,s3ObjectInfoToCopy, ObjectPath ):
        ''' Server Side Copy, eg upload report to latest, make copy to today'''
        self.s3client.copy_object(
            s3ObjectInfoToCopy.bucket_name,
            ObjectPath,
            CopySource(s3ObjectInfoToCopy.bucket_name, s3ObjectInfoToCopy.object_name),
           # metadata=metadata,
            metadata_directive=REPLACE,
        )

    #### Methods for a getting information using infrastructure information

    """ Method for gleaner store"""
    def listJsonld(self,bucket, repo, include_user_meta=False):
        """ urllist returns list of urn;s with urls"""
        # include user meta not working.
        path = f"{self.paths['summon']}/{repo}/"
        return self.listPath(bucket, path,include_user_meta=include_user_meta)

    def countJsonld(self,bucket, repo) -> int:
        count = len(list(self.listJsonld(bucket,repo)))
        return count

    def getJsonLD(self, bucket, repo, sha):
        path = f"{self.paths['summon']}/{repo}/{sha}.jsonld"
        s3ObjectInfo = {"bucket_name": bucket, "object_name": path}
        resp = self.getFileFromStore(s3ObjectInfo)
        return resp


    def listSummonedUrls(self,bucket, repo):
        """  returns list of urns with urls"""
        jsonlds = self.listJsonld(bucket, repo, include_user_meta=True)
        objs = map(lambda f: self.s3client.stat_object(f.bucket_name, f.object_name), jsonlds)
        o_list = list(map(lambda f: {"sha": shaFroms3Path(f.object_name),
                                     "url": f.metadata.get("X-Amz-Meta-Url"),
                                     "identifiertype": f.metadata.get("X-Amz-Meta-Identifiertype"),
                                     "lastmodified": f.last_modified
                                     }
                          , objs))
        return o_list

    def listSummonedSha(self,bucket, repo):
        """  returns list of urns with urls"""
        jsonlds = self.listJsonld(bucket, repo, include_user_meta=False)
        objs = map(lambda f: shaFroms3Path( f.object_name, extension=".jsonld"), jsonlds)
        # for ob in objs:
        #     print(ob)
        o_list = list(objs)
        return o_list

    def getJsonLDMetadata(self, bucket, repo, sha):
        path = f"{self.paths['summon']}/{repo}/{sha}.jsonld"
        s3ObjectInfo = {"bucket_name": bucket, "object_name": path}
        tags = self.getFileMetadataFromStore(s3ObjectInfo)

        return tags

    def getOringalUrl(self, bucket, repo, sha) -> str:
        md = self.getJsonLDMetadata(bucket, repo, sha)
        return md['Url']

    '''Cleans the name of slashes... might need more in the future.'''
    def getCleanObjectName(s3ObjectName) -> str:
        return s3ObjectName.replace('/','__')

    def listMilledRdf(self,bucket, repo,urnonly=False):
        path = f"{self.paths['milled']}/{repo}/"
        return self.listPath(bucket, path)
    def listMilledSha(self,bucket, repo,urnonly=False):
        paths = self.listMilledRdf(bucket,repo)
        shas = list(map(lambda p: shaFroms3Path(p.object_name, extension=".rdf"), paths))
        return shas

    def countMilledRdf(self,bucket, repo) -> int:
        count = len(list(self.listMilledRdf(bucket,repo)))
        return count
    ### methods for reporting
    '''
    Reporting will have to pull the original and put back to the datastore
    '''

    def listReportFile(self,bucket, repo,include_user_meta=False):
        """ urllist returns list of urn;s with urls"""
        # include user meta not working.
        path = f"{self.paths['report']}/{repo}/"
        return self.listPath(bucket, path,include_user_meta=include_user_meta)

    def putReportFile(self, bucket, repo, filename, json_str,  date="latest", copy_to_date=True):
        pass

    def putSitemapFile(self, bucket, repo, filename, sitemap):
        pass

    def getReportFile(self, bucket, repo, filename):
        path = f"{self.paths['report']}/{repo}/filename"
        s3ObjectInfo = {"bucket_name": bucket, "object_name": path}
        resp = self.getFileFromStore(s3ObjectInfo)
        return resp

    def getLatestRelaseUrl(self, bucket, source, extension='nq'):
        urls = self.getLatestRelaseUrls(bucket)
        url = pydash.find( urls, lambda x: source in x.get("object_name") and "_release" in x.get("object_name")  and x.get("object_name").endswith(extension) )
        return url.get('url')

    def getLatestRelaseUrls(self, bucket):
        path = f"{self.paths['release']}/latest/"
        urls = self.listPath(bucket,path)
        urls = map(lambda f: { "object_name": f.object_name,
                               "url": f"https://{self.endpoint}/{f.bucket_name}/{f.object_name}" }, urls)
        return list(urls)
    def getLatestSummaryUrl(self, bucket, source, extension='ttl'):
        urls = self.getLatestRelaseUrls(bucket)
        url = pydash.find( urls, lambda x: source in x.get("object_name") and "_release" in x.get("object_name")  and x.get("object_name").endswith(extension) )
        return url.get('url')

    def getLatestSummaryUrls(self, bucket):
        path = f"{self.paths['release']}/summary/"
        urls = self.listPath(bucket,path)
        urls = map(lambda f: { "object_name": f.object_name,
                               "url": f"https://{self.endpoint}/{f.bucket_name}/{f.object_name}" }, urls)
        return list(urls)

    def getRoCrateFile(self, filename, bucket="gleaner", user="public"):
        path = f"{self.paths['crate']}/{user}/{filename}"
        s3ObjectInfo = {"bucket_name": bucket, "object_name": path}
        resp = self.getFileFromStore(s3ObjectInfo)
        return resp

    def putRoCrateFile(self, data: str,filename, bucket="gleaner", user="public"):
        path = f"{self.paths['report']}/{user}/{filename}"
        s3ObjectInfo = {"bucket_name": bucket, "object_name": path}
        return self.putTextFileToStore(data, s3ObjectInfo)

    def getSitemapFile(self,bucket, repo, filename):
        path = f"{self.paths['sitemap']}/{repo}/filename"
        s3ObjectInfo = {"bucket_name": bucket, "object_name": path}
        resp = self.getFileFromStore(s3ObjectInfo)
        return resp

    def putSitemapFile(self,data: str, repo: str,filename: str, bucket="gleaner"):
        path = f"{self.paths['sitemap']}/{repo}/{filename}"
        s3ObjectInfo = {"bucket_name": bucket, "object_name": path}
        return self.putTextFileToStore(data, s3ObjectInfo)

"""
Basic abstraction, in case someone want to store files in a 
different method
"""
class MinioDatastore(bucketDatastore):
    """ Instance of a minio datastore with utility methods to retreive information"""
    def __init__(self, s3endpoint, options={},default_bucket="gleaner"):
        """ Initilize with
        Parameters:
            s3endpoint: endpoint. If this is aws, include the region. eg s3.us-west-2.amazon....
            options: creditials, other parameters to pass to client
            default_bucket: 'gleaner'
            """
        self.endpoint = s3endpoint
        self.options = {} if options is None else options # old code has none...
        self.default_bucket= default_bucket
        logging.info(str(options))
        self.s3client  =minio.Minio(s3endpoint, **self.options ) # this will neeed to be fixed with authentication


    def listPath(self, bucket, path, include_user_meta=False, recursive=True):
        """ returns the filelist for a path with the starting path removed from the list"""
        resp = self.s3client.list_objects(bucket, path, include_user_meta=include_user_meta, recursive=recursive)
        # the returned list includes the path
        o_list = filter(lambda f: f.object_name != path, resp)
        return o_list

    def listDuplicateUrls(self, bucket, sources, include_user_meta=False, recursive=False):
        if is_empty(sources):
            raise Exception("must provide sources")

        sources_to_run = sources
        dfs = pandas.DataFrame()
        paths = list(self.listPath(bucket, "summoned/", include_user_meta=include_user_meta, recursive=recursive))
        for p in paths:
            if not find(sources_to_run, lambda x: f"{self.paths.get('summon')}/{x}/" == p.object_name):
                continue
            try:
                jsonlds = self.listPath(bucket, p.object_name)
                objs = map(lambda f: self.s3client.stat_object(f.bucket_name, f.object_name), jsonlds)
                o_list = list(map(lambda f: {'Source': p.object_name,
                                             'Url': f.metadata.get('X-Amz-Meta-Url'),
                                             'Name': f.object_name,
                                             'Date': f.last_modified,
                                             }, objs))
            except Exception as e:
                logging.error(e)
            df = pandas.DataFrame(o_list)
            df['Url Duplicates'] = df.groupby(['Source', 'Url'])['Name'].transform('count')
            dfs = pandas.concat([dfs, df])
        return dfs


    def countPath(self, bucket, path):
        count = len(list(self.listPath(bucket, path)))
        return count

    def getFileFromStore(self, s3ObjectInfo):
        """ get an s3 file from teh store
        Parameters:
          s3ObjectInfo: {"bucket_name":obj.bucket_name, "object_name":obj.object_name }

        """
        resp = self.s3client.get_object(s3ObjectInfo["bucket_name"], s3ObjectInfo["object_name"])
        return resp.data
    def DataframeFromPath(self, bucket, path, include_user_meta=False):
        pathFiles = list(self.listPath(bucket,path,include_user_meta=include_user_meta))

        objs = map(lambda f: self.s3client.stat_object(f.bucket_name, f.object_name), pathFiles)
        data = map(lambda f: { "metadata": f.metadata, "bucket_name":f.bucket_name, "object_name":f.object_name}, objs )
        # does not work... should, but does not
        # data = list(map(lambda f:
        #                 pick(f,  'bucket_name', 'object_name')
        #                 , objs ))

        df = pandas.DataFrame(data=data)
        return df

    def getFileMetadataFromStore(self, s3ObjectInfo):
        """ get metadata s3 file from teh store
               Parameters:
                 s3ObjectInfo: {"bucket_name":obj.bucket_name, "object_name":obj.object_name }

               """
        s3obj = self.s3client.stat_object(s3ObjectInfo.get('bucket_name'), s3ObjectInfo.get('object_name'))
        md = s3obj.metadata
        user_meta = list()
        for o in md:
            if o.startswith("X-Amz-Meta"):
               user_meta.append({"name": o, "value": md[o]})
        # this needs to return the metadata
        return user_meta

    def putReportFile(self, bucket, repo, filename, json_str, date="latest", copy_to_date=True):
        path = f"{self.paths['report']}/{repo}/{date}/{filename}"
        f = BytesIO()
        length = f.write(bytes(json_str, 'utf-8'))
        f.seek(0)
        resp = self.s3client.put_object(bucket, path, f,length=length)
        if copy_to_date:
            today_str = datetime.now().strftime("%Y%m%d")
            path = f"{self.paths['report']}/{repo}/{today_str}/{filename}"
            self.copyObject(resp,path)
        return resp.bucket_name, resp.object_name

    def putSitemapFile(self, bucket, filename, json_str):
        path = f"{self.paths['sitemap']}/{filename}"
        f = BytesIO()
        length = f.write(bytes(json_str, 'utf-8'))
        f.seek(0)
        resp = self.s3client.put_object(bucket, path, f, length=length)
        return resp.bucket_name, resp.object_name

    def getReportFile(self, bucket, repo, filename):
        path = f"{self.paths['report']}/{repo}/latest/{filename}"
        s3ObjectInfo = {"bucket_name": bucket, "object_name": path}
        resp = self.getFileFromStore(s3ObjectInfo)
        return resp

    def getLatestRelaseFile(self, bucket, repo):
        path = f"{self.paths['graph']}/latest/summonded{repo}_latest_release.nq"
        s3ObjectInfo = {"bucket_name":bucket,"object_name": path}
        resp = self.getFileFromStore(s3ObjectInfo)
        return resp

    def getRelasePaths(self, bucket):
        path = f"{self.paths['graph']}/latest/"
        files = self.listPath(bucket, path)
        paths = list(map(lambda f:  f.object_name, files))
        return paths

    def getRoCrateFile(self, filename, bucket="gleaner", user="public"):
        path = f"/{self.paths['collection']}/{user}/{filename}"
        crate = self.s3client.get_object(bucket, path)
        return crate

    def removeObject(self, bucket, path):
        self.s3client.remove_object(bucket, path)
        return




def parts_from_urn(urn):
    parts = urn.split(':')
    id = parts[len(parts)-1]
    repo = parts[len(parts)-2]
    org = parts[len(parts)-3] ## new identifier will have an org identifier

    return {"id": id, "source":repo, "org":org, "original": urn}



#!/usr/bin/env python3

import argparse
import logging
import os
from ec.graph.manageGraph import ManageBlazegraph as mg
from ec.graph.sparql_query import _getSparqlFileFromResources
from ec.summarize.summarize_materializedview import summaryDF2ttl, get_summary4graph,get_summary4repoSubset
from ec.gleanerio.gleaner import endpointUpdateNamespace,getNabu, reviseNabuConfGraph, runNabu
from rdflib import Dataset, Namespace
from urllib.parse import urlparse
from ec.graph.release_graph import ReleaseGraph

def isValidURL(toValidate):
    o = urlparse(toValidate)
    if o.scheme and o.netloc:
        return True
    else:
        return False

def summarizeReleaseOnly():
    parser = argparse.ArgumentParser()
    parser.add_argument("--url", dest='url', help='url of file',
                        default="https://oss.geocodes-dev.earthcube.org/gleaner-wf/graphs/latest/summonediris_2023-03-13-11-02-47_release.nq"

                        )

    parser.add_argument("--repo", dest='repo', help='repo name used in the  urn')

    parser.add_argument('--s3base', dest='s3base',
                        help='basurl of',
                        default="https://oss.geocodes-dev.earthcube.org/"
                        )
    parser.add_argument('--s3bucket', dest='s3bucket',
                        help='basurl of',
                        default="gleaner-wf"

                        )
    parser.add_argument('--graphendpoint', dest='graphendpoint',
                        help='graph endpoint with namespace',
                        default="https://graph.geocodes-dev.earthcube.org/blazegraph/namespace/earthcube/sparql"
                        )
    parser.add_argument('--nographsummary', action='store_true', dest='nographsummary',
                        help='send triples to file', default=False)
    parser.add_argument('--summary_namespace', dest='summary_namespace',
                        help='summary_namespace defaults to {repo_summary}',
                        )
    args = parser.parse_args()

    repo = args.repo
    if args.summary_namespace:
        if isValidURL(args.summary_namespace):
            msg = 'For summary_namespace, Please enter the namespace only.'
            print(msg)
            logging.error(msg)
            return 1
        summary = args.summary_namespace
    else:
        summary = f"{repo}_summary"
    endpoint= args.graphendpoint
    graphendpoint = mg.graphFromEndpoint(endpoint)
    SCHEMAORG_http = Namespace("http://schema.org/")
    SCHEMAORG_https = Namespace("https://schema.org/")

    try:

        sumnsgraph = mg(graphendpoint, summary)

        summaryendpoint =endpointUpdateNamespace(endpoint,summary)

        rg = ReleaseGraph()
        rg.load_release(args.url)
        summarydf = rg.summarize()
        if len(summarydf) == 0:
            print("No result. Issue with RDF lib... use a triplestore")
            exit(1)

        nt,g = summaryDF2ttl(summarydf,repo) # let's try the new generator

        summaryttl = g.serialize(format='longturtle')
        # write to s3  in future
        # with open(os.path.join("output",f"{repo}.ttl"), 'w') as f:
        #      f.write(summaryttl)
        if not args.nographsummary:
            inserted = sumnsgraph.insert(bytes(summaryttl, 'utf-8'),content_type="application/x-turtle" )
            if inserted:
                logging.info(f"Inserted into graph store{sumnsgraph.namespace}" )
            else:
                logging.error(f" dumping file {repo}.ttl  Repo {repo} not inserted into {sumnsgraph.namespace}")

                with open(os.path.join("output",f"{repo}.ttl"), 'w') as f:
                     f.write(summaryttl)
                return 1
        else:
            logging.info(f" dumping file {repo}.ttl  nographsummary: {args.nographsummary} ")

            with open(os.path.join("output", f"{repo}.ttl"), 'w') as f:
                f.write(summaryttl)
    except Exception as ex:
        logging.error(f"error {ex}")
        print(f"Error: {ex}")
        return 1
    # finally:
    #     # need to figure out is this is run after return, I think it is.
    #     logging.debug(f"Deleting Temp namespace {tempnsgraph.namespace}")
    #     deleted = tempnsgraph.deleteNamespace()


if __name__ == '__main__':
    """ Summarize a from a gleaner 'release' set of n-quads file

    Description:
       untested
    """
    # these need to be better
    # url
    # OR
    # s3base, s3bucket
    #   if not repo, read all nq files in graphs/latest
    #
    # graph endpoint,

    print("This is not reliable. The RDF lib does not reliably generate the same reaults at Blazegraph")
    exitcode= summarizeReleaseOnly()
    exit(exitcode)



#!  python3
import argparse
from io import StringIO, BytesIO
import logging
import os
import sys

from ec.graph.sparql_query import queryWithSparql,listSparqlFilesFromResources

from ec.datastore import s3

logging.basicConfig(format='%(levelname)s : %(message)s', level=os.environ.get("LOGLEVEL", "INFO"), stream=sys.stdout)
log = logging.getLogger()

def basicQuery(args):
    """query an endpoint, return csv
    """
    log.info(f"Querying {args.graphendpoint} using {args.query}")
    parameters=None
    if args.urn:
        parameters={"urn":args.urn}
    if args.repo:
        parameters = {"repo": args.repo}
    counts = queryWithSparql(args.query,  args.graphendpoint, parameters=parameters)

    csv_result = counts.to_csv( quoting=1, index=False)
    if (args.output):
        args.output.write(csv_result)
    else:
        print (csv_result)
    return 0
def start():
    """
        Run the query_graph program.
        A tool to use the queries in the earthcube utilities.
        Arguments:
            args: Arguments passed from the command line.
        Returns:
            Result as csv file.

    """
    description="A tool to use the queries in the earthcube utilities. You can only use the all_ queries, at present"
    epilog = "Queries \n" + '"'+ '", \n "'.join(listSparqlFilesFromResources()) +'"'

    parser = argparse.ArgumentParser(description=description, epilog=epilog)

    parser.add_argument("query", help='select_one')
    parser.add_argument('--graphendpoint', dest='graphendpoint',
                        help='graph endpoint' ,default="https://graph.geocodes-dev.earthcube.org/blazegraph/namespace/earthcube/")
    parser.add_argument("--output", type=argparse.FileType('w'), help='output file')
    parser.add_argument("--repo", dest="repo", help='value to pass to a repo_ query')
    parser.add_argument("--urn", dest="urn", help='urn/graph identidier to urn_  query')

    args = parser.parse_args()
    exitcode = basicQuery(args)


if __name__ == '__main__':
    start()



#!/usr/bin/env python3

import argparse
import logging
import os
import errno
from ec.graph.manageGraph import ManageBlazegraph as mg
from ec.summarize import summaryDF2ttl, get_summary4graph,get_summary4repoSubset
from ec.gleanerio.gleaner import endpointUpdateNamespace,getNabu, reviseNabuConfGraph, runNabu
from urllib.parse import urlparse


def dumpToFile(repo,summaryttl ):
    if repo is None or repo == '':
        repo = 'pass_repo_flag'
    filename = os.path.join("output", f"{repo}.ttl")
    if not os.path.exists(os.path.dirname(filename)):
        try:
            os.makedirs(os.path.dirname(filename))
        except OSError as exc:  # Guard against race condition
            if exc.errno != errno.EEXIST:
                raise
    with open(filename, 'w') as f:
        f.write(summaryttl)
    return
def isValidURL(toValidate):
    o = urlparse(toValidate)
    if o.scheme and o.netloc:
        return True
    else:
        return False


def summarizeGraphOnly():
    """ Summarize directly from a namespace, upload to provided summarize namespace

    Description:
        * query for repository graphs using defined repo parameter,

        * build summary triples

        * loading to a summarized namespace

    """
    parser = argparse.ArgumentParser()

    parser.add_argument("--repo", dest='repo', help='repo name used in the  urn')

    parser.add_argument('--graphendpoint', dest='graphendpoint',
                        help='graph endpoint with namespace',
                        default="https://graph.geocodes-dev.earthcube.org/blazegraph/namespace/earthcube/sparql"
                        , required=True)

    parser.add_argument('--nographsummary', action='store_true', dest='nographsummary',
                        help='send triples to file', default=False)
    parser.add_argument('--summary_namespace', dest='summary_namespace',
                        help='summary_namespace  just the namspace. defaults to "repo_summary"',
                        )
    args = parser.parse_args()
    repo = args.repo

    if args.summary_namespace:
        if isValidURL(args.summary_namespace):
            msg = 'For summary_namespace, Please enter the namespace only.'
            print(msg)
            logging.error(msg)
            return 1
        summary = args.summary_namespace
    else:
        summary = f"{repo}_summary"

    endpoint= args.graphendpoint
    graphendpoint = mg.graphFromEndpoint(endpoint)

    try:

        sumnsgraph = mg(graphendpoint, summary)

        summaryendpoint =endpointUpdateNamespace(endpoint,summary)

        if repo is not None:
            summarydf = get_summary4repoSubset(endpoint, repo)
        else:
            # this really needs to be paged ;)
            summarydf = get_summary4graph(endpoint)
            repo = ""

        nt,g = summaryDF2ttl(summarydf,repo) # let's try the new generator

        summaryttl = g.serialize(format='longturtle')
        # write to s3  in future
        # with open(os.path.join("output",f"{repo}.ttl"), 'w') as f:
        #      f.write(summaryttl)
        if not args.nographsummary:
            inserted = sumnsgraph.insert(bytes(summaryttl, 'utf-8'),content_type="application/x-turtle" )
            if inserted:
                logging.info(f"Inserted into graph store{sumnsgraph.namespace}" )
            else:
                logging.error(f" dumping file {repo}.ttl  Repo {repo} not inserted into {sumnsgraph.namespace}")
                dumpToFile(repo, summaryttl)
                return 1
        else:
            logging.info(f" dumping file {repo}.ttl  nographsummary: {args.nographsummary} ")

            dumpToFile(repo, summaryttl)
    except Exception as ex:
        logging.error(f"error {ex}")
        print(f"Error: {ex}")
        return 1

if __name__ == '__main__':

    exitcode= summarizeGraphOnly()
    exit(exitcode)



#!  python3
import click
import logging
import json
import sys

from pydash.collections import find
from pydash import is_empty
from ec.gleanerio.gleaner import getSitemapSourcesFromGleaner, getGleaner
from ec.reporting.report import missingReport
from ec.datastore import s3

@click.command()
@click.option('--cfgfile', help='gleaner config file', type=click.Path(exists=True))
@click.option('--graphendpoint', help='graph endpoint'
              )
# no default for s3 parameters here. read from gleaner. if provided, these override the gleaner config
@click.option('--s3server', help='s3 server address')
@click.option('--s3bucket', help='s3 bucket')
@click.option('--no_upload', help='do not upload to s3 bucket',is_flag=True, default=False)
@click.option('--output', help='dump to file', type=click.File('wb'))
@click.option('--source', help='gone or more repositories (--source a --source b)', multiple=True)
@click.option('--milled', help='include milled', is_flag=True,default=False)
@click.option('--summon', help='check summon only',is_flag=True, default=False)

def writeMissingReport(cfgfile, graphendpoint, s3server, s3bucket, no_upload, output, source, milled, summon):
    if cfgfile:
        s3endpoint, bucket, glnr = getGleaner(cfgfile)
        minio = glnr.get("minio")
        # passed paramters override the config parameters
        s3server = s3server if s3server else s3endpoint
        bucket = s3bucket if s3bucket else bucket
    else:
        s3server = s3server
        bucket = s3bucket
    graphendpoint = graphendpoint  # not in gleaner file, at present
    if is_empty(s3server) or is_empty(bucket):
        logging.fatal(f" must provide a gleaner config or (s3endpoint and s3bucket)]")
        return 1
    if is_empty(graphendpoint) and not summon:
        logging.fatal(f" must provide graphendpoint if you are checking milled")
        return 1
    logging.info(f" s3server: {s3server} bucket:{bucket} graph:{graphendpoint}")
    s3Minio = s3.MinioDatastore(s3server, None)
    sources = getSitemapSourcesFromGleaner(cfgfile)
    sources = list(filter(lambda source: source.get('active'), sources))
    sources_to_run = source  # optional if null, run all

    for i in sources:
        source_url = i.get('url')
        source_name = i.get('name')
        if sources_to_run is not None and len(sources_to_run) >0:
            if not find (sources_to_run, lambda x: x == source_name ):
                continue
        try:
            report = missingReport(source_url, bucket, source_name, s3Minio, graphendpoint, milled=milled, summon=summon)
            report = json.dumps(report,  indent=2)
            if output:  # just append the json files to one filem, for now.
                logging.info(f" report for {source_name} appended to file")
                output.write(report)
            if not no_upload:
                s3Minio.putReportFile(bucket, source_name, "missing_report.json", report)
        except Exception as e:
            logging.error(f"could not write missing report for {source_name} to s3server:{s3server}:{bucket} error:{str(e)}")
    return 0


def start():
    """
        Run the write_missing_report program.
        Get a list of active repositories from the gleaner file.
        For each repository, generate missing reports and write these information to a json file and upload it to s3.
        Arguments:
            args: Arguments passed from the command line.
        Returns:
            An exit code.
    """
    exitcode = writeMissingReport()
    sys.exit(exitcode)


if __name__ == '__main__':
    start()




import csv
from io import StringIO
from string import Template

import pandas
from rdflib import Namespace, Dataset,Graph, ConjunctiveGraph
from rdflib.namespace import RDF

from  ec.graph.sparql_query import _getSparqlFileFromResources
from ec.datastore.s3 import MinioDatastore


# notes... we can group concat where a field is possibly none. we have to use a bind.
# from placenames and keywords

SCHEMAORG_http = Namespace("http://schema.org/")
SCHEMAORG_https = Namespace("https://schema.org/")
class ReleaseGraph:
   # dataset = Dataset(default_union=True,store="Oxigraph")
    dataset = Dataset(default_union=True)
    dataset.bind('schema_http',SCHEMAORG_http)
    dataset.bind('schema', SCHEMAORG_https)
    #dataset = ConjunctiveGraph()
    filename = ""

    def load_release(self, file_or_url, format='nquads' ):
        self.dataset.parse(file_or_url, format=format)
    def read_release(self, s3server, s3bucket, source, date="latest", options={}):
        s3 = MinioDatastore(s3server, options)
        url = s3.getLatestRelaseUrl(s3bucket, source)
        self.filename = url[ url.rfind('/') +1 :]
        self.load_release(url)

    def summarize(self):
        print("SUMMARIZE from RELEASE not reliable. produces different results than blazegrapn (no min/max depth")
        # get the summary sparql query, run it sparql data frome to put it in a dataframe
        #might just feed the result rows to pandas
        # all_summary_query returns no rows ;)
       # resource = ec.graph.sparql_query._getSparqlFileFromResources("all_summary_query")
       # resource = ec.graph.sparql_query._getSparqlFileFromResources("all_repo_count_datasets")
        # result = self.dataset.query(resource)

        #result = self.dataset.query(test_types, initNs={'schema_o': SCHEMAORG_http, 'schema':SCHEMAORG_https })
        query = _getSparqlFileFromResources('all_summary_query')
       # result = self.dataset.query(summary_sparql, result='sparql', initNs={'schema_old': SCHEMAORG_http, 'schema': SCHEMAORG_https})
        result = self.dataset.query(query, result='sparql')

        #result = self.dataset.query(summary_sparql)
        csvres = result.serialize(format="csv")
        csvres = csvres.decode()
        csv_io = StringIO(csvres)
        df = pandas.read_csv(csv_io)
        return df

    def query_release(self, template_name='all_summary_query',parameters={}):
        query = _getSparqlFileFromResources(f"{template_name}")
        # get the summary sparql query, run it sparql data frome to put it in a dataframe
        #might just feed the result rows to pandas
        # all_summary_query returns no rows ;)
       # resource = ec.graph.sparql_query._getSparqlFileFromResources("all_summary_query")
       # resource = ec.graph.sparql_query._getSparqlFileFromResources("all_repo_count_datasets")
        # result = self.dataset.query(resource)
        q_template = Template(query)
        thsGraphQuery = q_template.substitute(parameters)
        #result = self.dataset.query(test_types, initNs={'schema_o': SCHEMAORG_http, 'schema':SCHEMAORG_https })
        result = self.dataset.query(thsGraphQuery, result='sparql', initNs={'schema_old': SCHEMAORG_http, 'schema': SCHEMAORG_https})
        #result = self.dataset.query(summary_sparql)
        csvres = result.serialize(format="csv")
        csvres = csvres.decode()
        csv_io = StringIO(csvres)
        df = pandas.read_csv(csv_io)
        return df
# types works, summary does not.



"""
This basically wraps sparqldataframe,
and contains a way to get resources that are the sparql queries,
and few helpers to basic queries
"""
import pandas
from  pydash import  ends_with, replace_end, sort
import sparqldataframe
from string import Template
try:
    import importlib.resources as pkg_resources
except ImportError:
    # Try backported to PY<37 `importlib_resources`.
    import importlib_resources as pkg_resources

import ec.graph.sparql_files as sparqlfiles

"""
QUery with a spaql query

parameters are an object so to pass in a reponame:
{"repo": self.repo}

or to pass in a graph
{"g": self.repo}
"""

def queryWithSparql( template_name : str, endpoint : str,parameters:object={}) -> pandas.DataFrame:
    """ Query a SPARQL endpoint, and return a Pandas Dataframe

    Parameters:
       template_name: name of templates in the ec.graph.sparql_files directory
       endpoint: SPARQL endpoint url
       parameters: object with the names to fill in template eg {"repo": "reponame"}
    """
    query = _getSparqlFileFromResources(f"{template_name}")
    q_template = Template(query)
    thsGraphQuery = q_template.substitute(parameters)
    q_df = sparqldataframe.query(endpoint, thsGraphQuery)
    return q_df

## this will need to be done to package specifications.
# https://stackoverflow.com/questions/6028000/how-to-read-a-static-file-from-inside-a-python-package
def _getSparqlFileFromResources(filename) -> str:
    """ retrieves sparql file from the sparql_files folder when in a package"""
    resourcename = f"{filename}.sparql"
    resource = pkg_resources.read_text(sparqlfiles, resourcename)
    return resource
    # with open(f"./resources/{filename}", "r") as stream:
    #     try:
    #         return stream.read()
    #     except Exception as exc:
    #         print(exc)
def listSparqlFilesFromResources() -> str:
    """ retrieves sparql file from the sparql_files folder when in a package"""
    resource = pkg_resources.contents(sparqlfiles)
    files = filter( lambda f: ends_with(f, ".sparql"), resource)
    files = map(lambda f: replace_end(f,".sparql",""), files)
    files = sort(list(files))
    return files

def getAGraph(  g, endpoint: str) -> pandas.DataFrame:
    """Query a SPARQL endpoint and return a Pandas Dataframe for a geocodes object"""
    query = _getSparqlFileFromResources('urn_triples_for_a_graph')
    q_template = Template(query)
    thsGraphQuery = q_template.substitute(urn=g)
    g_df = sparqldataframe.query(endpoint, thsGraphQuery)

    return g_df



import csv
import logging
from io import StringIO

import advertools as adv
import numpy as np
import pandas
from numpy import ndarray
from pandarallel import pandarallel
from pandas import DataFrame

import requests, sys, os
import yaml
from pandas.core.arrays import ExtensionArray

def _urlExists(sitemapurl):
    try:
        r = requests.get(sitemapurl)
        if r.status_code == 404:
            logging.error("Sitemap URL is 404")
            return False
    except:
        return False
    return True
def _urlResponse(item_loc: str):
    """return response code, content type, """
    try:
        r = requests.head(item_loc)
        if r.status_code != 200:
            return [r.status_code, None]
        else:
            content_type = r.headers.get("Content-Type")
            return [r.status_code, content_type]
    except:
        return [400, None]
class Sitemap():
    """ This holds information about a sitemap, and allows for testing of the
    URLS in the sitemap to see if they exist.
    If the sitemap URL does not exist, then it will return a Sitemap, with an
    len(errors) > 0.

    """
    sitemapurl = None
    _validSitemap = True
    sitemap_df = pandas.DataFrame()
    _checkedUrls=False
    def __init__(self, sitemapurl, repoName="", no_progress_bar=False):
        self.errors=[]
        self.sitemapurl = sitemapurl
        self.no_progress_bar = no_progress_bar
        if _urlExists(sitemapurl):
            self.sitemap_df = adv.sitemap_to_df(sitemapurl)
            self._validSitemap = True
        else:
            self.errors.append(f"sitemap url invalid: {sitemapurl}")
            self._validSitemap = False
            # the other option is to return None.

    def validUrl(self) -> bool:
        """ does the provided sitemap URL exist."""
        return self._validSitemap

    def errors(self):
        return self.errors
    def uniqueItems(self):
        """list of unqiue sitemaps records"""
        return self.sitemap_df.sitemap.unique().tolist()

    def uniqueUrls(self) :
        """Returns a pandas series of the URLS'"""
        return self.sitemap_df["loc"].unique().tolist()

    def check_urls(self) -> DataFrame:
        """This will run head on the list of url's in the pandas dataframe.
        It will append columns ("url_response","content_type") to the sitemap_df
        """

        # add columns to the dataframe. clear out any previous values
        df = self.sitemap_df
        df["url_response"]=None
        df["content_type"]=None

        if not self._validSitemap:
            self._checkedUrls = True
            return df

        pandarallel.initialize(progress_bar=self.no_progress_bar)
        df["url_response"],df["content_type"]=  zip(*df.parallel_apply(lambda row:
                           _urlResponse(row.get('loc')),
                           axis=1))
        self._checkedUrls =True
        return df

    def get_url_report(self) -> str:
        """returns a csv of the 'loc','lastmod','url_response', 'content_type' """
        if not self._checkedUrls:
            self.check_urls()
        out= StringIO()
        self.sitemap_df.to_csv(out, index=False, quoting=csv.QUOTE_ALL,
                columns=['loc','lastmod','url_response', 'content_type']
              #                 columns=['loc','lastmod',"url_response/content_type"]
                               )
        return out.getvalue()

class GleanerioSourceSitemap(Sitemap):
    """ For a provided GleanerIO source record, create a Sitemap object to utilize"""
    source= None

    def __init__(self, source):
        self.self.source = source
        if source["sourcetype"] !=  "sitemap":
            return Exception("source is Not a sitemap")
        super.__init__(source.url, reponame= source.name)

    def get_status(self):
        s = self.source
        return  { 'name': s.name , 'code': _urlResponse( s.get("url") ), 'description': "res", 'url': s.get("description"), 'type': s.get("sourcetype")}



from typing import Union

import pandas
import json
from string import Template

from pyld import jsonld
from rdflib import URIRef, BNode, Literal, Graph, Dataset

# this context will need to be expanded.
from .utils import compact_jld_str, formatted_jsonld

from ec.graph.sparql_query import getAGraph



def is_http(u: str) -> bool:
    if not isinstance(u, str) :
        print("might need to set LD_cache") #have this where predicate called
        return None
    #might also check that the str has no spaces in it,&warn/die if it does
    return u.startswith("http")

def createRDFNode(nodeValue: str) -> Union[Literal, BNode ,URIRef]:
    "fix_url and quote otherwise"
    if not isinstance(nodeValue,str):
        if  (nodeValue is None) or  (pandas.isnull(nodeValue)):
            return Literal("")
        return Literal(nodeValue)
    else:
        if nodeValue.startswith("<ht"):
            return URIRef(nodeValue)
        elif nodeValue.startswith("_:B"):
            return BNode(nodeValue.replace("_:B", "B"))
        elif nodeValue.startswith("t1"):
            return BNode(nodeValue.replace("t1", "Bt1"))
        elif is_http(nodeValue):
            return URIRef(nodeValue)
        elif nodeValue.startswith("doi:"):
            return URIRef(nodeValue)
        elif nodeValue.startswith("DOI:"):
            return URIRef(nodeValue)
        #elif obj:
        elif nodeValue is None:
            return Literal("")
        elif pandas.isnull(nodeValue):
            return Literal("")
        else:
            # import json
            # return json.dumps(url)
           return Literal(nodeValue)
    #else:
    #    return url

def df2rdfgraph(df: pandas.DataFrame):
    "print out df as .nt file"

    g = Graph()
    g.bind("schema", "https://schema.org/")
    for index, row in df.iterrows():
        s=df["s"][index]
        s=createRDFNode(s)
        p=df["p"][index]
        p=createRDFNode(p)
        o=df["o"][index]
        o=createRDFNode(o)
        g.add((s, p, o))

        #need to finish up w/dumping to a file
    return  g


def get_rdfgraph(urn: str, endpoint: str ) -> Graph: #get graph
    df=getAGraph(urn, endpoint)
    g=df2rdfgraph(df)
    return g

def graph2jsonld(g, form="jsonld", schemaType="Dataset") -> str:
    """get jsonld from endpoint

    Parameters:
        g: ?g from sparql query. URN of the graph eg. urn:gleaner.io:earthcube:geocodes_demo_datasets:257108e0760f96ef7a480e1d357bcf8720cd11e4
        form: jsonld| compact, frame
        schemaType: if form=frame then this type is passed to the frame
    """
    # auto_compact=False might change
    jld_str = g.serialize(format="json-ld")

    return formatted_jsonld(jld_str, form=form, schemaType=schemaType)

# returns a framd JSON
# form= framed|compact
def get_graph2jsonld(urn: str, endpoint:str, form="compact", schemaType="Dataset") -> str:
    """get jsonld from endpoint

    Parameters:
        urn: ?g from sparql query. URN of the graph eg. urn:gleaner.io:earthcube:geocodes_demo_datasets:257108e0760f96ef7a480e1d357bcf8720cd11e4
        form: jsonld| compact, frame
        schemaType: if form=frame then this type is passed to the frame
        endpoint: sparql endpoint

    """
    g = get_rdfgraph(urn, endpoint)

    return graph2jsonld(g, form=form, schemaType=schemaType)


def get_rdf2jld_str(urn: str, endpoint:str) -> str:
    "get jsonld from endpoint"
    g= get_rdfgraph(urn, endpoint)
    jld_str = g.serialize(format="json-ld")
    return compact_jld_str(jld_str)

####
def load_release(releaseurl:str) -> Graph:
    """retrieves a release from the url"""
    g= Dataset()
    g.parse(releaseurl, format='nquads')
    return g
#  using https://github.com/cadmiumkitty/rdfpandas
    #g = Graph()
#    g.parse(releaseurl, format='nt')
#    df = to_dataframe(g)



import json
import logging
from datetime import date, datetime
import time
from typing import Any

import pandas
import pydash

import ec.sitemap
from ec.graph.release_graph import ReleaseGraph
from ec.graph.sparql_query import queryWithSparql

from ec.datastore.s3 import bucketDatastore
from ec.sitemap import Sitemap

import csv
from urllib import request


"""
reports

simplifying the thinking.
These go into a reports store in the datastore

we can calculate in multiple ways
eg for items that were not summoned due to no jsonld, calculate from s3 compare to sitemap, and pull for gleaner logs 

Let's start with ones we can do easily.

Reports
* PROCESSING REPORT: (processing.json) 
**  general report with the basics. counts, good, bad, etc.
*** sitemap count
*** summoned count ec.datastore.s3.countJsonld
*** milled count ec.datastore.s3.countMilled
*** graph count repo_count_graphs.sparql ec.graph.sparql_query.queryWithSparql("repo_count_graphs", release_file, parameters={"repo": repo})
*** when processing details is working, then add counts of  was summoned but did not make it into the graph

* PROCESSING REPORT DETAILS:
** thought... how to handle what got lost... need to know, or perhaps files with lists of what got lost along the way
*** SITEMAP Details and issues
**** (sitemap_badurls.csv)list of bad urls
**** (sitemap_summon_issues.csv) list of urls for items that had no JSONLD. 
*****  Grab list of metadater-Url from Datastore, ec.datastore.s3.listSummonedUrls
*****  compare to sitemap url list
*****  remove bad urls. if it cannot be retrieved, we don't need to chase it down
*** PROCESSING Detials and issues
**** (summon_graph_missing.csv; summon_milled_missing.csv;) what made and did not make it. Parameters
**** summoned ids: ec.datastore.s3.listJsonld
# will need to do a list(map(lambda , collection) to get a list of urls.
o_list = list(map(lambda f: ec.datastore.s3.urnFroms3Path(f.object_name), objs))
**** milled ids: ec.datastore.s3.listMilledRdf
**** graph ids:  ec.graph.sparql_query.queryWithSparql("repo_select_graphs", release_file, parameters={"repo": repo})

***** suggest compare using pydash, or use pandas...
then look up the urls' using: ec.datastore.s3.getJsonLDMetadata

* GRAPHSTORE REPORTS: sparql.json
This runs a list of sparql queries
** What is in the overall graph, 
** Data Loading reports by  Repo 


Probably run the all repo report monthly, or after a large data load
Run the repo report have a repo is reloaded.

FUTURE:
Use issues from repo reports as data for a tool that can evaluate the failures

"""
def get_url_from_sha_list(shas: list,  bucket, repo, datastore: bucketDatastore):
    """if a repo has issues with milling or loading to a graph, then get the summoned file"""

    pass

def missingReport(valid_sitemap_url :str , bucket, repo, datastore: bucketDatastore, graphendpoint, milled=True, summon=False):
    today = date.today().strftime("%Y-%m-%d")
    response = {"source":repo,"graph":graphendpoint,"sitemap":valid_sitemap_url,
                "date": today, "bucket": bucket, "s3store": datastore.endpoint }
    t = time.time()
    sitemap = ec.sitemap.Sitemap(valid_sitemap_url)
    if not sitemap.validUrl():
        raise ValueError(valid_sitemap_url)
    sitemap_urls = sitemap.uniqueUrls()
    response["sitemap_geturls_time"] = time.time() - t
    sitemap_count = pydash.collections.size(sitemap_urls)
    t = time.time()
    summoned_list = datastore.listSummonedUrls(bucket, repo)
    response["s3_geturls_time"] = time.time() - t
    summoned_count = pydash.collections.size(summoned_list)
    summoned_urls = list(map(lambda s: s.get("url"), summoned_list))
    dif_sm_summon = pydash.arrays.difference(sitemap_urls, summoned_urls)
    dif_summon_sm = pydash.arrays.difference( summoned_urls, sitemap_urls)
    response["sitemap_count"] = sitemap_count
    response["summoned_count"] = summoned_count
    response["missing_sitemap_summon_count"] = len(dif_sm_summon)
    response["missing_sitemap_summon"] = dif_sm_summon
    response["extra_in_summon_count"] = len(dif_summon_sm)
    response["extra_in_summon"] = dif_summon_sm
    if summon:
        return response
    ##### summmon to graph
    t = time.time()
    summoned_sha_list = datastore.listSummonedSha(bucket, repo)
    response["summon_list_s3_sha_time"] = time.time() - t
    t = time.time()
    graph_urns = ec.graph.sparql_query.queryWithSparql("repo_select_graphs", graphendpoint, {"repo": repo})
    graph_shas = list(map(lambda u: pydash.strings.substr_right_end(u, ":"), graph_urns['g']))
    response["graph_sha_urn_time"] = time.time() - t
    dif_summon_graph = pydash.arrays.difference(summoned_sha_list, graph_shas)
    response["graph_urn_count"] = pydash.collections.size(graph_shas)
    response["missing_summon_graph_count"] = len(dif_summon_graph)
    response["missing_summon_graph"] = dif_summon_graph
    if milled:
        t = time.time()
        milled_list = datastore.listMilledSha(bucket, repo)
        response["milled_sha_time"] = time.time() - t
        dif_summon_milled = pydash.arrays.difference(summoned_sha_list, milled_list)
        response["milled_count"] = pydash.collections.size(milled_list)
        response["missing_summon_milled"] = len(dif_summon_milled)
        response["missing_summon_milled"] = dif_summon_milled
    return response

def compareSitemap2Summoned(valid_sitemap_url :str , bucket, repo, datastore: bucketDatastore):
    #Grab list of metadater-Url from Datastore, ec.datastore.s3.listSummonedUrls
    sitemap = ec.sitemap.Sitemap(valid_sitemap_url)
    sitemap_urls = sitemap.uniqueUrls()
    sitemap_count= pydash.collections.size(sitemap_urls)
    summoned_list = datastore.listSummonedUrls(bucket,repo)
    summoned_count = pydash.collections.size(summoned_list)
    summoned_urls=list(map(lambda s: s.get("url"),summoned_list ))
    difference = pydash.arrays.difference( sitemap_urls, summoned_urls)
    return {
        "sitemap_count": sitemap_count,
        "summoned_count": summoned_count,
            "missing": difference
            }


def compareSummoned2Milled(bucket, repo, datastore: bucketDatastore):
    """ return list of missing urns/urls
    Generating milled will be good to catch such errors"""
    # compare using s3, listJsonld(bucket, repo) to  listMilledRdf(bucket, repo)
    summoned_list = datastore.listSummonedSha(bucket, repo)
    milled_list = datastore.listMilledSha(bucket, repo)
    difference = pydash.arrays.difference(summoned_list, milled_list)
    return {
        "summoned_count": pydash.collections.size(summoned_list),
        "milled_count": pydash.collections.size(milled_list),
            "missing": difference
            }
def compareSummoned2Graph(bucket, repo, datastore: bucketDatastore, graphendpoint):
    """ return list of missing .
    we do not alway generate a milled.
    """
    # compare using s3, listJsonld(bucket, repo) to queryWithSparql("repo_select_graphs", release_file)
    summoned_list = datastore.listSummonedSha(bucket, repo)
    graph_urns = ec.graph.sparql_query.queryWithSparql("repo_select_graphs",graphendpoint,{"repo":repo})
    graph_shas = list(map(lambda u: pydash.strings.substr_right_end(u, ":"), graph_urns['g']))
    difference = pydash.arrays.difference(summoned_list, graph_shas)
    return {
        "summoned_count": pydash.collections.size(summoned_list),
        "milled_count": pydash.collections.size(graph_shas),
            "missing": difference
            }


##################################
#  REPORT GENERATION USING SPARQL QUERIES
#   this uses defined spaql queries to return counts for reports
###################################
reportTypes = {
    "all": [
        {"code": "triple_count", "name": "all_count_triples"},
            {"code": "graph_count_by_repo", "name": "all_repo_count_graphs"},
        {"code": "dataset_count", "name": "all_count_datasets"},
        {"code": "dataset_count_by_repo", "name": "all_repo_count_datasets"},
        {"code": "types_count", "name": "all_count_types"},
        {"code": "types_count_by_repo", "name": "all_repo_count_types"},
        {"code": "mutilple_version_count", "name": "all_count_multiple_versioned_datasets"},
        {"code": "mutilple_version_count_by_repo", "name": "all_repo_count_versioned_datasets"},
        {"code": "repos_with_keywords", "name": "all_repo_with_keywords"},
    ],
    # add the triple count by graph, and graph sizes
    # this will need to be added, managed in the generate_graph
    # add a basic by default, detailed if requested with a flag
    "all_detailed": [
        {"code": "triple_count", "name": "all_count_triples"},
        {"code": "graph_count_by_repo", "name": "all_repo_count_graphs"},
        {"code": "dataset_count", "name": "all_count_datasets"},
        {"code": "dataset_count_by_repo", "name": "all_repo_count_datasets"},
        {"code": "types_count", "name": "all_count_types"},
        {"code": "types_count_by_repo", "name": "all_repo_count_types"},
        {"code": "mutilple_version_count", "name": "all_count_multiple_versioned_datasets"},
        {"code": "mutilple_version_count_by_repo", "name": "all_repo_count_versioned_datasets"},
        {"code": "keywords_counts_by_repo", "name": "all_repo_count_keywords"},
        {"code": "keywords_count", "name": "all_count_keywords"},
        {"code": "variablename_count", "name": "all_count_variablename"},
        {"code": "graph_sizes", "name": "all_graph_sizes"},

    ],
    "repo": [
        {"code": "triple_count", "name": "repo_count_triples"},
        {"code": "dataset_count", "name": "repo_count_datasets"},
        {"code": "type_count", "name": "repo_count_types"},
        {"code": "kw_count", "name": "repo_count_keywords"},
        {"code": "variablename_count", "name": "repo_count_variablename"},
        {"code": "version_count", "name": "repo_count_multi_versioned_datasets"},
        {"code": "graph_sizes_count", "name": "repo_graph_sizes"},
    ],
    # add the triple count by graph, and graph sizes
    # this will need to be added, managed in the generate_graph
    # add a basic by default, detailed if requested with a flag
    "repo_detailed": [
        {"code": "triple_count", "name": "repo_count_triples"},
        {"code": "kw_count", "name": "repo_count_keywords"},
        {"code": "dataset_count", "name": "repo_count_datasets"},
        {"code": "triple_count", "name": "repo_count_triples"},
        {"code": "type_count", "name": "repo_count_types"},
        {"code": "version_count", "name": "repo_count_multi_versioned_datasets"},
        {"code": "variablename_count", "name": "repo_count_variablename"},
        {"code": "graph_sizes_count", "name": "repo_graph_sizes"},
        {"code": "triple_count_by_graph", "name": "repo_count_triples_by_graph"},
    ]
}

def _get_report_type(reports, code) -> str:
    report = pydash.find(reports, lambda r: r["code"] == code)
    return report["name"]

# this is a sinlge repo...
# was reportTypes["all"]
# really don't even need to do the filers, but
# repo works best for now
def generateGraphReportsRelease(repo,  release_file, reportList=reportTypes["repo"]) -> Any:
    #queryWithSparql("repo_count_types", release_file)
    parameters = {"repo": repo}
    current_dateTime = datetime.now().strftime("%Y-%m-%d")
    rg = ReleaseGraph()
    rg.load_release(release_file)
    reports= []
    for report in reportList:
        try:
            t = time.time()
            result =  rg.query_release(template_name=report["name"],parameters=parameters)
            elapsed_time = time.time() - t
            data = result.dropna()
            data = data.to_dict('records')
            reports.append(  {"report": report["code"],
                     "processing_time": elapsed_time,
                     "length": len(data),
             "data": data
             })
        except Exception as ex:
            logging.error(f"query with sparql against release failed: report:{report['code']}  repo:{repo}   {ex}")
            elapsed_time = time.time() - t
            reports.append( {"report": report["code"],
                    "errpr": f"{ex}",
                     "processing_time": elapsed_time,
             "data": []
             })
    return json.dumps({"version": 0, "repo": repo, "date": current_dateTime, "reports": reports }, indent=4)

##  for the 'object reports, we should have a set.these could probably be make a set of methos with (ObjectType[triples,keywords, types, authors, etc], repo, endpoint/datastore)
def generateGraphReportsRepo(repo, graphendpoint, reportList=reportTypes["all"]) -> str:
    current_dateTime = datetime.now().strftime("%Y-%m-%d")
    reports = map (lambda r:    generateAGraphReportsRepo(repo, r,
                                 graphendpoint, reportList)
                                      ,
                                reportList)

    reports = list(reports)
    return json.dumps({"version": 0, "repo": repo, "date": current_dateTime, "reports": reports }, indent=4)


def generateAGraphReportsRepo(repo, r, graphendpoint, reportList) -> Any:
    #queryWithSparql("repo_count_types", release_file)
    parameters = {"repo": repo}
    try:
        t = time.time()
        report =   queryWithSparql(_get_report_type(reportList, r['code']), graphendpoint, parameters=parameters)
        elapsed_time = time.time() - t
        data = report.dropna()
        data = data.to_dict('records')

        return  {"report": r["code"],
                 "processing_time": elapsed_time,
                 "length": len(data),
         "data": data
         }
    except Exception as ex:
        logging.error(f"query with sparql failed: report:{r['code']}  repo:{repo}   {ex}")
        elapsed_time = time.time() - t
        return {"report": r["code"],
                "errpr": f"{ex}",
                 "processing_time": elapsed_time,
         "data": []
         }

def getGraphReportsLatestRepoReports(repo,  datastore: bucketDatastore):
    """get the latest for a dashboard"""
    date="latest"
    path = f"{datastore.paths['reports']}/{repo}/{date}/sparql.json"
    filelist = datastore.getReportFile(datastore.default_bucket, repo, path)

def listGraphReportDates4Repo(repo,  datastore: bucketDatastore):
    """get the latest for a dashboard"""
    path = f"{datastore.paths['reports']}/{repo}/"
    filelist = datastore.listPath(path)
    return filelist

def generateIdentifierRepo(repo, bucket, datastore: bucketDatastore):
    jsonlds = datastore.listJsonld(bucket, repo, include_user_meta=True)
    objs = map(lambda f: datastore.s3client.stat_object(f.bucket_name, f.object_name), jsonlds)
    o_list = list(map(lambda f: {'Source': repo,
                                 'Identifiertype': f.metadata.get('X-Amz-Meta-Identifiertype'),
                                 'Matchedpath': f.metadata.get('X-Amz-Meta-Matchedpath'),
                                 'Uniqueid': f.metadata.get('X-Amz-Meta-Uniqueid'),
                                 'Example': f.metadata.get('X-Amz-Meta-Uniqueid')
                                 }, objs))
    df = pandas.DataFrame(o_list)
    identifier_stats = df.groupby(['Source', 'Identifiertype', 'Matchedpath'], group_keys=True, dropna=False) \
        .agg({'Uniqueid': 'count', 'Example': lambda x: x.iloc[0:5].tolist()}).reset_index()
    return identifier_stats

# Example of CSV URL: 'https://docs.google.com/spreadsheets/d/e/2PACX-1vTt_45dYd5LMFK9Qm_lCg6P7YxG-ae0GZEtrHMZmNbI-y5tVDd
# 8ZLqnEeIAa-SVTSztejfZeN6xmRZF/pub?gid=1340502269&single=true&output=csv'
def readSourceCSV(csv_url):
    response = request.urlopen(csv_url)
    csv_reader = csv.DictReader(response.read().decode('utf-8').splitlines())
    data_list = list(csv_reader)
    return data_list

def generateReportStats(url, bucket, datastore: bucketDatastore, graphendpoint, community):
    sources = readSourceCSV(url)
    if community != "all":
        sources = list(filter(lambda source: community in source.get('Community'), sources))
        if len(sources) == 0:
            return None
    else:
        sources = list(filter(lambda source: source.get('Active') == "TRUE", sources))

    # graphendpoint needs to be summary for this sparql
    df = queryWithSparql("repo_count_graphs_summary", graphendpoint)
    # dropping null value columns to avoid errors
    df.dropna(inplace=True)
    df["repo"] = df["g"].str.split(":", n=-1, expand=True)[3]
    df = df.groupby(["repo"])["g"].nunique().reset_index(name='DistinctCount')

    report = []
    for i in sources:
        source_url = i.get('URL')
        source_landing_page = i.get('Domain')
        source_name = i.get('Name')
        source_proper_name = i.get('ProperName')
        source_community = i.get('Community')
        source_des = i.get('Description')

        df_repo = df[df["repo"] == source_name]
        try:
            sm = Sitemap(source_url)
            if not sm.validUrl():
                logging.error(f"Invalid or unreachable URL: {source_url} ")

            source_records = 0
            if df_repo.empty:
                logging.info(f"Repo is empty in graph: {source_name} ")
            else:
                source_records = df_repo["DistinctCount"].values[0].astype(str)

            dict = {
                "source": source_name,
                "title": source_proper_name,
                "website": source_landing_page,
                "sitemap": source_url,
                "image": f"{source_name}.png",
                "community": source_community,
                "description": source_des,
                "records": source_records
            }

            report.append(dict)

        except Exception as e:
            logging.error(
                f"could not write report stats for {source_name} {bucket} error:{str(e)}")

    report_json = json.dumps(report, indent=4)

    return report_json







#!/usr/bin/env python3

import argparse
import errno

import logging
import os

from ec.graph.manageGraph import ManageBlazegraph as mg
from ec.summarize.summarize_materializedview import summaryDF2ttl, get_summary4repo
from ec.gleanerio.gleaner import endpointUpdateNamespace, getNabu, reviseNabuConfGraph, runNabu, getNabuFromFile
from urllib.parse import urlparse



def dumpToFile(repo,summaryttl ):
    filename = os.path.join("output", f"{repo}.ttl")
    if not os.path.exists(os.path.dirname(filename)):
        try:
            os.makedirs(os.path.dirname(filename))
        except OSError as exc:  # Guard against race condition
            if exc.errno != errno.EEXIST:
                raise
    with open(filename, 'w') as f:
        f.write(summaryttl)
    return
def isValidURL(toValidate):
    o = urlparse(toValidate)
    if o.scheme and o.netloc:
        return True
    else:
        return False


def summarizeRepo():
    """ Summarize a repository using a temporary graph namespace

    Description:
        * read nabu config,

        * uploading to a graph namespace

        * building summarize triples

        * loading to a summarized namespace

    """
    parser = argparse.ArgumentParser()

    parser.add_argument("repo", help='repository name')
    parser.add_argument('nabufile', type=argparse.FileType('r'),
                        help='nabu configuration file')
    parser.add_argument('--graphendpoint', dest='graphendpoint',
                        help='use this endpoint (full url:https://graph.geocodes-dev.earthcube.org/blazegraph/namespace/earthcube/sparql"). overrides nabu endpoint')
    parser.add_argument('--glcon', dest='glcon',
                        help='override path to glcon', default="~/indexing/glcon")
    parser.add_argument('--nographsummary', action='store_true', dest='nographsummary',
                        help='send triples to file', default=False)
    parser.add_argument('--keeptemp', dest='graphtemp',
                        help='do not delete the temp namespace. a namespace "{repo}_temp" will be created', default=True)
    parser.add_argument('--summary_namespace', dest='summary_namespace',
                        help='summary_namespace. just the namepsace defaults to "{repo}_temp_summary"')
    args = parser.parse_args()

    repo = args.repo
    if args.summary_namespace:
        if isValidURL(args.summary_namespace):
            msg = 'For summary_namespace, Please enter the namespace only.'
            print(msg)
            logging.error(msg)
            return 1
        summary = args.summary_namespace
    else:
        summary = f"{repo}__temp_summary"
    nabucfg = args.nabufile
    endpoint, cfg = getNabuFromFile(nabucfg)
    graphendpoint = mg.graphFromEndpoint(endpoint)
    tempnsgraph = mg(graphendpoint, f'{repo}_temp')
    try:  # temp has been created
        created = tempnsgraph.createNamespace()
        if ( created=='Failed'):
            logging.fatal("coould not create namespace")
        sumnsgraph = mg(graphendpoint, summary)
        created = sumnsgraph.createNamespace()
        if ( created=='Failed'):
            logging.fatal("coould not create summary namespace")
        # endpoints for file
        tempendpoint =endpointUpdateNamespace(endpoint,f"{repo}_temp")
        summaryendpoint =endpointUpdateNamespace(endpoint,summary)
        newNabucfg = reviseNabuConfGraph(cfg, tempendpoint)
        runNabu(newNabucfg,repo, args.glcon )

        summarydf = get_summary4repo(tempendpoint)
        nt,g = summaryDF2ttl(summarydf,repo) # let's try the new generator
        summaryttl = g.serialize(format='longturtle')
        # write to s3  in future
        dumpToFile(repo, summaryttl)
        if not args.nographsummary:
            inserted = sumnsgraph.insert(bytes(summaryttl, 'utf-8'),content_type="application/x-turtle" )
            if inserted:
                logging.info(f"Inserted into graph store{sumnsgraph.namespace}" )
            else:
                logging.error(f" dumping file {repo}_dumped.ttl  Repo {repo} not inserted into {sumnsgraph.namespace}")
                dumpToFile(f"{repo}_dumped", summaryttl)
                return 1
    except Exception as ex:
        logging.error(f"error {ex}")
        print(f"Error: {ex}")
        return 1
    finally:
        # need to figure out is this is run after return, I think it is.
        if not args.graphtemp :
            logging.debug(f"Deleting Temp namespace {tempnsgraph.namespace}")
            deleted = tempnsgraph.deleteNamespace()
        return 0

if __name__ == '__main__':
    logging.info("Not Ready")
    exitcode= summarizeRepo()
    exit(exitcode)



from io import StringIO

from ec.sitemap.sitemap import Sitemap
import argparse

from ec.logger import config_app

log = config_app()

def sitemap_checker(args):
    log.debug(" Sitemap")
    """Sitemap checker. Default option  checks if url is sitemap exist"""
    sitemap = Sitemap(args.sitemapurl, no_progress_bar=args.no_progress)
    if  not args.nocheck :
        sitemap.check_urls()

    return sitemap.get_url_report()

def start():
    """
        Run the sitemap_checker program.
        Sitemap checker. Default option  checks if url is sitemap exist.
        Arguments:
            args: Arguments passed from the command line.
        Returns:
            result of check as csv.

    """
    parser = argparse.ArgumentParser()
    parser.add_argument("sitemapurl", help='sitemapurl')
    parser.add_argument("--output", type=argparse.FileType('w'), help='output file')
    parser.add_argument("--no-url-check",action='store_true', dest="nocheck" ,help='output file', default=False)
    parser.add_argument("--no-progress", action='store_false',   dest="no_progress" ,help='no progress bar', default=True)
    args = parser.parse_args()

    result = sitemap_checker(args)
    if args.output:
        log.info(" Sitemap written to file"  )
        args.output.write(result)
    else:
        print(result)

if __name__ == '__main__':

    result = start()





class ec_object():
    schematype='Dataset'
    schema_obj = None # pyld object

    # need to have a frame
    def __init__(self, urn, object_manager,schematype='Dataset'):
        self.schematype = schematype


    def to_rdf_graph(self):
        pass
    def to_rdf_dataset(self):
        pass

    def to_nquads(self):
        pass
    def to_jsonld(self, form='frame'):
        pass